      PROGRAM new_hyperz_v12

c     Version v12.2

c  This program computes, in addition to the usual hyperz issues, 
C  the following integrated probabilities:
c  the probability of an object to be at z_cluster +/-0.05 and +/-0.10  
c  and integrated P(z) at z +/-0.1, between z=z1-z2, per bin in dz fixed by the user 
C  A new filters file is used to deal with HR around e-lines when computing 
c  the "filter" function.
c  Several absolute magnitudes M_lambda are provided instead of a single one in the past.  
c  UP TO N<=10 additional filters for absolute magnitudes are considered  
c  Includes the possibility of reading (and thus appliying) a correction E(B-V) per object.
c  An extra column is added to .z_phot with the INTEGRATED probability within z_best+/-0.1, 
c  normalized between z_min and z_max
c.. If a file with "reference" ASCII templates is given (E/S0-Sbc-Scd-Im-SB), 
c.. Ex. reference_spectra.param,
c.. 6 extra columns are given for the best-fit template at z_best, 
c.. and the reduced xi2 for each "reference" template. 
c.. PARAMETER FILE: new_hyperz.param

c     Version 7.4 : 31/3/2006: R. Pello
c  Correction bugs on P_int
c  Corrected bug on integrated P(z)
c  Corrected bug on .obs-sed when output in AB or microJy
c  Corrected bug on .spe & .m

c     Version 8.0 : 5/Apr/2006: R. Pello
c  Including modifications suggested by D. Schaerer
c  Warning when template numbers exceed maximum mxtyp
c  Corrected bug on z=-9.00 type objects

c     Version 9.0 : 23/Apr/2006: R. Pello
c  Including additional scaling of "b" parameter translated into physical units:
c  *.obs_sed: f_lambda (Observed)= b * 2x10^(-17) * F_lambda(model) (erg/s/A/cm2)
c  Internal units:
c    F_lambda (intrinsic) = b_scale * F_lambda (model)  (b_scale: physical scaling)
c    F_lambda (intrinsic)/ [(1+z)*4*pi*d_lum^2] = b * 2x10^(-17) * F_lambda(model)
c              b_scale = b * 2x10^(-17) * [(1+z) * 4 * pi * d_lum^2] 
c              b_scale : direct model scaling WHEN MODEL MAKES SENSE 
c Ex: Bruzual & Charlot 
c        - Bursts & e-folding models are given in solar luminosities/ solar_Mass
c          with L_solar=3.90 10^(33) erg/sec
c          => b_scale ~ solar_Masses * 3.90 10^(33) typically (b_scale ~ up to 10^(43)!)
c        - CSFR models are normalized to 1 solar_Mass/year
c          => b_scale ~ few units
c TO AVOID INF PROBLEMS, b_scale is given in log(b_scale) units

c  Version : 9.1
c     We can read Starburst99 models: flag S

c Version: 10.0 JUL 06
c     Bug on abs. mags M_abs corrected
c     Option for M_abs calculations, either 
c 1) Scaled to the first "reference" filter (old option in v8 and v9), or
c 2) Global scaling according to b_scale, irrespective of the "reference" filter
c Version: 10.0 AUG 06
c   Bug on filter corrections c_corr(i) when input m_AB 

c Version: 10_CFHTLS NOV 06
c Additionnal output: ****.PDZ, with a line per object

c Version: 3 APR 07
C Fixed bugs as for new_hyper_fit_v10.f
c Corrected bug on search for maxima : Searching minima of chi2 (l=1869)
c Corrected potential bug when nebv>0 and no individual E(B-V) column is provided

c  Version: 29 MAY 07
c  Corrected bug in PDZ normalization for z outside the permitted M_abs constraints

c  Version 10/05/07: Double Real*8 b_scale
c  Version 13/06/07: corrected error in b_scale: (1+z) factor correction not needed
c  ("filter" routine provides the correct scaling)

c  Version 10/09/07: Merging CFHTLS and normal version.
c                    *.PDZ file become an optional output
c                    Bug corrected: normalization P_int and P(z_cluster)+/-0.05/0.1
c                                   when z/z_cluster <0.05 or 0.1                     
c
c  Version 19/09/07: Combination of filters with T(lambda) in energy AND in photons 
c                    within the same catalog. An additional parameter is given for each filter 
c                    in c6 of the filter's file: 1 => energy; 2=> photons(default)

c Version 28/09/2008: Correcting for undersampling in templates (.spe and .temp_sed)

c Version 18/06/2009: Reading Maraston templates (type "MA")

c Version  1/09/2011: Including an optional "soft" prior in luminosity using a Schecter luminosity prior
c                     applied on P(z) (also output by .PDZ and .log_phot)

c Version  1/01/2013: Update LF prior using a Schecter luminosity function applied on P(z) 
c                     (also output by .PDZ and .log_phot) with correct volume weighting
c                     New output :  median P(z) distribution 

c Version  7/11/2013: Including a *.header_zphot file 
c                     Corrected bug on Starbursts99 ages

      IMPLICIT NONE
c
      INTEGER i,ii,iii,j,k,l,ks,n_files,icomp,ncomp,kfil,iw_vega,iw,
     .     n_filter,iob,nob,id,iout_s,iout_m,law_rd,nfil,iroot,matrix,
     .     iw_ob,i_typ,i_red,i_age,i_lyf,nz,itype,iage,ilyf,nlyf,izmin,
     .     itypem,iagem,iredm,ilyfm,npoiz,n_sol,iwav,npoiz1,npoiz2,
     .     last,nred,ired,law_nd,i_lyman,mxz,izsol,mxage,mxtyp,mxfil,
     .     mxred,mxlyf,mxwl,ndeg,magtyp,iunit_sed,i1,i2,icat,iblank,
     .     input,nparam,ipass,izmin1,iagemax,iread,jread,nf,np,mxft,
     .     mxwf,nfilt_mabs,iz_clus,n_fil_wa,n_fil_w,itypea,ireda,
     .     iagea,ilyfa,iztyp,idum,izphot,i_sol,iout,iz,iforbid,n_head,
     .     iz_clus05a,iz_clus05b,iz_clus10a,iz_clus10b,iz1,iz2,nfmab,
     .     ncomp2,kfil2,ncol,n_fil_abs,n_aux,nlimit,nebv,last_k,
     .     iz_int10a,iz_int10b,iz_top10a,iz_top10b,trans_type,
     .     n_ref,n_files_ref,iw_ref,iz_ref,mscale,ntop1,ntop2,m,ncomp3
      REAL w_vega,f_vega,T_red,f_red,f,w,z1,z2,amag,err_p,mag_nd,
     .     err_nd,err_min,avmin,avmax,averr,wl,surf,c_corr,val,p,
     .     err_fl,dz,zmax,zmin,zz,z_min_array,z,y,zop,flux,filter,
     .     av,q_pro,dz_f,zmin_f,zmax_f,age,fl,qq,poids,abmag,abmag2,
     .     poids_z,q_pro1,q_pro2,q_proz,chi2_b1,chi2_b2,z_b,z_b1,
     .     z_b2,dd,z_poids,qq_p,w0,wmin1,wmax1,wmin0,wmax0,wsyn,ysyn,
     .     d1,d2,yob,zz1,zz2,t,vega,pthresh,conv_mag,bp1,cputime,
     .     time_tot,part_time,f_jy,err_jy,deg,pchi2,qchi2,
     .     zero,chi2,r_chi2,amabs_1,amabs_2,sauce,amagb,bmag,dif_w,
     .     dif_w_min,fl_temp,mag_temp,func,prob,err,x1,x2,ww,h0,om,ov,
     .     ageuni,tz,r,zeropoint,d_mod,color,z_clus,amagb_clus,p_clus,
     .     dummy,dummyz,zphot,chi2sol,qq2,fl_nd,fl1,ebv_mw,av_mw,rlin,
     .     fl_ext,chi2min,bb_array,c,fnu_vega,rnu_vega,wrnu,rrnu,
     .     f1,f2,ffnu,trapz1,fact_lyf,r_lyf,renor_log,p05,p10,proba,
     .     p_Ediscs,z_Ediscs,zp1,zp2,dzp,ebv_in,aw_in,p_int,proba_z,
     .     pro_z,av_clus,d_lum,pi,ppp,t_sb,dd2,count,afl,count_max,
     .     w_ref,flux_ref,f_ref,xi2_ref,test_age,vol,E,al_star,alpha,
     .     prior
      REAL*8 bm,wt,sxx,sxy,b,bb,ddummy,b_scale
c
      PARAMETER (err=1.E-8,nparam=44,mxft=300,mxwf=200)
      INCLUDE 'dimension.dec'

      LOGICAL header
      CHARACTER header_line*5000
      CHARACTER*160 command_name
      INTEGER sys_return
      CHARACTER NAMECH(mxtyp)*160,sed_abc(mxtyp)*2,line*240
      CHARACTER*160 spectra,amatrix,filedef,filein,filt_file,
     .     cat_in,spectra1,spectra2,spectra3,spectra4,spectra5,
     .     spec_vega,filtresfile,line_sed,filterout,copyright1,
     .     copyright2,copyright3,spectra6,file_PDZ,file_header,
     .     filedef_ref,NAMECH_ref(mxtyp)
      CHARACTER sed_abc_ref(mxtyp)*2
      CHARACTER clog*1,ccat*1,cz*1,csed1*1,csed2*1,cage*1,cout_m*1,
     .     cPDZ,cout_s*1,root*8,date*24,vv*5
      CHARACTER fmt1*90,fmt2*90,fmt3*90,fmt4*90,fmt6*90,fmtPDZ*90
      CHARACTER*500 fmt2009,fmt2008
      CHARACTER*500 fmt2001
      CHARACTER*40 cname,param(nparam)
c
      DIMENSION input(nparam),kfil(mxfil),nfilt_mabs(mxfil)
      DIMENSION t_red(225),t(225),f_red(mxwl,225,mxtyp),ks(mxtyp)
      DIMENSION w_vega(4000),f_vega(4000),vega(0:mxfil)
      DIMENSION bp1(0:mxfil),conv_mag(0:mxfil)
      DIMENSION c_corr(mxfil),fl_nd(mxfil),fl1(mxfil)
      DIMENSION f_jy(mxfil),err_jy(mxfil),fl_temp(mxfil),mag_temp(mxfil)
      DIMENSION amag(mxfil),err_p(mxfil),abmag(mxz)
      DIMENSION mag_nd(mxfil),err_nd(mxfil),law_nd(mxfil)
      DIMENSION ysyn(mxwl),wsyn(mxwl),ww(mxwl),yob(mxwl),y(mxwl)
      DIMENSION z_min_array(mxz),p_Ediscs(mxz),z_Ediscs(mxz)
      DIMENSION i_typ(mxz),i_age(mxz),i_red(mxz),i_lyf(mxz)
      DIMENSION wl(0:mxfil),flux(0:mxfil,0:mxz,mxage,mxtyp,mxred,mxlyf)
      DIMENSION q_pro(mxz),q_pro1(mxz),q_pro2(mxz),q_proz(mxz)
      DIMENSION z(0:mxz),z_b(mxz),z_b1(mxz),z_b2(mxz)
      DIMENSION chi2_b1(mxz),chi2_b2(mxz),iforbid(mxz),bb_array(mxz)
      DIMENSION izphot(mxz),zphot(mxz),bb(mxz),chi2sol(mxz),n_aux(100)
      DIMENSION val(3),zz1(3),zz2(3),p(3),amagb(mxfil),ncomp2(mxfil)
      DIMENSION zeropoint(0:mxfil),n_fil_w(mxz),kfil2(mxfil,mxfil)
      DIMENSION fnu_vega(4000),rnu_vega(4000),n_fil_abs(mxfil,mxz)
      DIMENSION wrnu(mxwf),rrnu(mxwf),ffnu(mxwf),abmag2(mxfil,mxz)
      DIMENSION proba_z(mxz),vol(mxz),prior(mxz)
      DIMENSION flux_ref(0:mxfil,0:mxz,mxtyp,mxlyf),xi2_ref(mxtyp)
      DIMENSION F_ref(mxwl,2,mxtyp),w_ref(mxwl,mxtyp),iw_ref(mxtyp)
      DIMENSION t_sb(37),trans_type(mxfil)
c
      EXTERNAL cputime,rlin,filter,pchi2,qchi2,zero,func,tz,trapz1,
     .         pro_z
      EXTERNAL d_mod,color
      COMMON/et1/surf(0:mxfil)
      COMMON/et2/W(mxwl,mxtyp),F(mxwl,mxage,mxtyp),fl(mxfil),
     .     err_fl(mxfil)
      COMMON/et3/law_rd
      COMMON/et4/n_filter,nfil(0:mxfil),iw(mxtyp),i_lyman(mxfil)
      COMMON/et5/bm
      COMMON/compose/icomp
      COMMON/nfree/deg,prob
      COMMON/filterres/filtresfile
      COMMON/c_filter/iread,jread,nf,np(mxft)
      COMMON/c_filter2/r(mxwf,mxft,2)
      COMMON/lyf_sigma/fact_lyf
      data t_sb/0.0,0.001,0.002,0.003,0.004,0.005,0.006,0.007,0.008,
     .0.009,0.010,0.011,0.012,0.013,0.014,0.015,0.016,0.017,0.018,
     .0.019,0.020,0.03,0.04,0.05,0.06,0.07,0.08,0.09,0.10,0.2,0.3,
     .0.4,0.5,0.6,0.7,0.8,0.9/

c_________________________________________________________________________

      vv='12.2 '
      copyright1='New-hyperz photometric redshift code - Version '//vv
     .     //' (Nov, 7 2013 - R. Pello-)' 
      copyright2=' revised version of hyperz, by Micol Bolzonella, '','
     .     //' Roser Pello, Joan-Marc Miralles'
      copyright3=' More information at '
     .     //'http://webast.ast.obs-mip.fr/hyperz/'
      WRITE(*,*)
      WRITE(*,'(a100)') copyright1
      WRITE(*,*)
      WRITE(*,'(a100)') copyright2
      WRITE(*,'(a100)') copyright3
      WRITE(*,*)

      jread=0                   ! Control on reading A0VSED
      iread=0                   ! Control on reading Filter Library
      c=2.9979E+18              ! speed of light in AA/s
      nebv=0                    ! Control on reading individual E(B-V)
      pi=3.14159265358979       ! PI constant
      mscale=1                  ! Default value of M_abs scaling option

c ..... Probabilities for the confidence intervals
      p(1)=0.99
      p(2)=0.90
      p(3)=0.68
      x1=0.
      x2=200.
      ndeg=1
      deg=REAL(ndeg)
      DO i=1,3
         prob=p(i)
         val(i)=zero(func,x1,x2,err)  ! chi2 for P = p(i) 
      END DO

c....... limits in absolute magnitude & default lum prior settings

      amabs_1=-100.0
      amabs_2=100.0
      z_clus=-1.
      al_star=1.0
      alpha=1.0


 11   filein='new_hyperz.param'
      last=INDEX(filein,'  ')-1
      WRITE(*,*) 'Name of parameter file'
      WRITE(*,*) '[default = ',filein(1:last),']'
      READ(*,'(A)') line
      IF (line.ne.' ') THEN
         READ(line,'(A)',ERR=11) filein
      END IF
      time_tot=cputime()
c ------------------------------------------------------------------------
c                       Reading parameters file 
c ------------------------------------------------------------------------

c ..... Parameters in common hyperz & make_catalog
      param(1)='A0VSED'
      param(2)='FILTERS_RES'
      param(3)='FILTERS_FILE'
      param(4)='TEMPLATES_FILE'
      param(5)='MAG_TYPE'
      param(6)='ERR_MAG_MIN'
      param(7)='Z_MIN'
      param(8)='Z_MAX'
      param(9)='REDDENING_LAW'
      param(10)='AV_MIN'
      param(11)='AV_MAX'
      param(12)='LY_FOREST'
      param(13)='OUTPUT_FILE'
      param(14)='H0'
      param(15)='OMEGA_M'
      param(16)='OMEGA_V'
c ..... Parameters relative to hyperz only
      param(17)='FILT_M_ABS'
      param(18)='CATALOG_FILE'
      param(19)='CATALOG_TYPE'
      param(20)='Z_STEP'
      param(21)='ZSTEP_TYPE'
      param(22)='AV_STEP'
      param(23)='AGE_CHECK'
      param(24)='PROB_THRESH'
      param(25)='OUTPUT_TYPE'
      param(26)='SED_OBS_FILE'
      param(27)='SED_TEMP_FILE'
      param(28)='LOGPHOT_FILE'
      param(29)='CATPHOT_FILE'
      param(30)='ZPHOT_FILE'
c ..... Optional parameters
      param(31)='Z_CLUSTER'
      param(32)='M_ABS_MIN'
      param(33)='M_ABS_MAX'
      param(34)='MATRIX'
      param(35)='SPECTRUM'
      param(36)='EBV_MW'
c ..... New optional parameters (EDiscs P_int)
      param(37)='ZP1'
      param(38)='ZP2'
c ..... New optional parameters (adaptive E(B-V); user gives n_column)
      param(39)='EBV_unit'
c ..... New optional parameter ("Reference" ASCII templates)
      param(40)='REF_TEMPLATES_FILE'
c ..... New optional parameter (Scaling option for M_abs)
      param(41)='MSCALE_FILT_M_ABS'
c ..... New optional parameter (PDZ file)
      param(42)='PDZ_FILE'
c ..... New optional parameter (M_STAR, Schechter M* for Lum prior)
      param(43)='M_STAR'
c ..... New optional parameter (ALPHA,  Schechter alpha for Lum prior)
      param(44)='ALPHA'
c............................
c............................

      DO i=1,nparam
         input(i)=0
      END DO
      ebv_mw=0.

      OPEN(77,FILE=filein,STATUS='old',ERR=11)
      WRITE(*,*) 'Reading parameter file :'

 7    READ(77,'(A)',ERR=10,END=10) line
      DO i=1,80
         IF (line(i:i).ne.' ') THEN
            line=line(i:)
            GO TO 8
         END IF
      END DO
      GO TO 7  ! blank line
 8    IF (line(1:1).eq.'#') GO TO 7 ! comment line
      iblank=INDEX(line,' ')
      cname=line(1:iblank-1) ! variable name
      line=line(iblank:)
      DO i=1,80
         IF (line(i:i).ne.' ') THEN
            line=line(i:)
            IF (line(1:1).eq.'#') THEN
               WRITE(*,*) 'Missed value of variable ', cname
               STOP
            END IF
            GO TO 9
         END IF
      END DO
      WRITE(*,*) 'Missed value of variable ', cname
      STOP
 9    iblank=INDEX(line,' ')
      line=line(1:iblank-1) ! part of string containing value of variable
c Parameters in common hyperz & make_catalog
c ..... File containing Vega spectrum
      IF (cname.eq.param(1)) THEN
         READ(line,'(A)') spec_vega
         input(1)=1
         GO TO 7
c ..... File FILTER.RES
      ELSE IF (cname.eq.param(2)) THEN
         READ(line,'(A)') filtresfile
         input(2)=1
         GO TO 7
c ..... File containing filter parameters and instructions for 
c       non detected objects (nd) or out of file (oof) for each filter
c       0 -> flux = -1 (filter not considered) for either nd or oof
c       1 -> flux = -1 for oof, flux = 0, err_flux = flux_nd for nd
c       2 -> flux = -1 for oof, flux = flux_nd/2, err_flux = flux_nd/2 for nd
c       3 -> flux = -1 for oof, mag = mag_nd,  err = err_nd for nd
      ELSE IF (cname.eq.param(3)) THEN
         READ(line,'(A)') filt_file
         input(3)=1
         GO TO 7
c ..... File containing synthetic spectra filenames
      ELSE IF (cname.eq.param(4)) THEN
         READ(line,'(A)') filedef
         input(4)=1
         GO TO 7
c ..... Magnitude type:
c       0 -> standard magnitudes
c       1 -> AB magnitudes
      ELSE IF (cname.eq.param(5)) THEN
         READ(line,*) magtyp
         IF (magtyp.ne.0.and.magtyp.ne.1) THEN
            WRITE(*,*) 'MAG_TYPE must be 0 or 1'
            STOP
         END IF
         input(5)=1
         GO TO 7
c ..... Minimum error in magnitude (if error < err_min => err = err_min)
      ELSE IF (cname.eq.param(6)) THEN
         READ(line,*) err_min
         input(6)=1
         GO TO 7
c ..... Redshift range (utilized only if icat=0) and step for scan
      ELSE IF (cname.eq.param(7)) THEN
         READ(line,*) zmin
         input(7)=1
         GO TO 7
      ELSE IF (cname.eq.param(8)) THEN
         READ(line,*) zmax
         input(8)=1
         GO TO 7
c ..... Reddening:
c       0 -> No reddening
c       1 -> Allen, 1976, standard MW
c       2 -> Seaton, 1979, standard MW
c       3 -> Fitzpatrick, 1986, LMC
c       4 -> Prevot, 1984 + Bouchet, 1985, SMC
c       5 -> Calzetti et al., 2000
c       6 -> Cardelli et al., 1989
      ELSE IF (cname.eq.param(9)) THEN
         READ(line,*) law_rd
         IF (law_rd.ne.0.and.law_rd.ne.1.and.law_rd.ne.2.and.
     .        law_rd.ne.3.and.law_rd.ne.4.and.law_rd.ne.5.
     .        and.law_rd.ne.6) THEN
            WRITE(*,*) 'REDDENING_LAW must be 0 -- 6'
            STOP
         END IF
         input(9)=1
         GO TO 7
c ..... Av_max, Av_min
      ELSE IF (cname.eq.param(10)) THEN
         READ(line,*) avmin
         input(10)=1
         GO TO 7
      ELSE IF (cname.eq.param(11)) THEN
         READ(line,*) avmax
         input(11)=1
         GO TO 7
c .... Lyman Forest
      ELSE IF (cname.eq.param(12)) THEN
         READ(line,*) r_lyf
         IF (r_lyf.lt.1.e-4) r_lyf=1.
         IF (r_lyf.lt.1.) r_lyf=1./r_lyf
         IF (ABS(r_lyf-1.).lt.1.e-4) THEN
            nlyf=1 ! considering only the mean Madau value
         ELSE
            nlyf=3 ! considering 3 values: mean Madau value + 2 values 
         END IF
         input(12)=1
         GO TO 7
c ..... Output files: no extension (file = *)
c       *.cat_phot -> catalogue
c       *.log_phot -> log file
c       *.sed      -> SED
c       *.z_phot   -> catalogue with only first maximum
      ELSE IF (cname.eq.param(13)) THEN
         READ(line,'(A)') spectra
         iblank=INDEX(spectra,' ')
         spectra1=spectra(1:iblank-1)//'.cat_phot'
         spectra2=spectra(1:iblank-1)//'.log_phot'
         spectra3=spectra(1:iblank-1)//'.obs_sed'
         spectra4=spectra(1:iblank-1)//'.z_phot'
         spectra5=spectra(1:iblank-1)//'.temp_sed'
         spectra6=spectra(1:iblank-1)//'.proba_int'
         filterout=spectra(1:iblank-1)//'.out_phot'
c Version CFHTLS: P(z) line per object in *.PDZ
         file_PDZ=spectra(1:iblank-1)//'.PDZ'
c Header file : *.header_zphot
         file_header=spectra(1:iblank-1)//'.header_zphot'
         input(13)=1
         GO TO 7
c ..... Cosmological parameters to compute the universe age and M_abs 
      ELSE IF (cname.eq.param(14)) THEN
         READ(line,*) h0
         input(14)=1
         GO TO 7
      ELSE IF (cname.eq.param(15)) THEN
         READ(line,*) om
         input(15)=1
         GO TO 7
      ELSE IF (cname.eq.param(16)) THEN
         READ(line,*) ov
         input(16)=1
         GO TO 7
c Parameters relative to hyperz only
c ..... Filter for the absolute magnitude
      ELSE IF (cname.eq.param(17)) THEN
         input(17)=1
c read filters on a line sepparated by comas (composite filters allowed)
         do i=1,10
         ncomp2(i)=0
         enddo
         i=1
         ncol=1
         ncomp2(1)=1   ! at least 1 reference filter
         DO k=1,80
         IF (line(k:k).eq.'&') THEN ! first filter composite
         line(k:k)=' '
         ncomp2(i)=ncomp2(i)+1
         END IF
         IF (line(k:k).eq.',') THEN !starting new filter
         line(k:k)=' '
         i=i+1
         ncomp2(i)=ncomp2(i)+1
         ncol=1
         END IF
         enddo

         nfmab=i  ! number of filters for absolute mags

c        READ(line,*) nfilt_mabs

         nlimit=0
         do i=1,nfmab
         nlimit=nlimit+ncomp2(i)
         enddo
         READ(line,*)(n_aux(i),i=1,nlimit)
         write(*,*)'There are ',nfmab,' filters for absolute mags:'
         
         j=1
         do i=1,nfmab
         do ii=1,ncomp2(i)
         kfil2(i,ii)=n_aux(j)
         j=j+1
         enddo
         write(*,*)i,(kfil2(i,ii),ii=1,ncomp2(i))
         enddo

         GO TO 7
c...... Options for M_abs scaling
      ELSE IF (cname.eq.param(41)) THEN
         READ(line,*)mscale
         IF (mscale.ne.1.and.mscale.ne.2) THEN
            WRITE(*,*) 'MSCALE_FILT_M_ABS must be 1 or 2'
            STOP
         END IF
         input(41)=1
         GO TO 7
c ..... File containing photometric catalogue
      ELSE IF (cname.eq.param(18)) THEN
         READ(line,'(A)') cat_in
         input(18)=1
         GO TO 7
c ..... Catalogue format 
c       Format of objects catalogue: 
c       N,[z_min,z_max,](mag(i),i=1,n),(err(i),i=1,n)
      ELSE IF (cname.eq.param(19)) THEN
         READ(line,*) icat
         IF (icat.ne.0.and.icat.ne.1) THEN
            WRITE(*,*) 'CATALOG_TYPE must be 0 or 1'
            STOP
         END IF
         input(19)=1
         GO TO 7
c ..... Redshift step used to build hypercube
      ELSE IF (cname.eq.param(20)) THEN
         READ(line,*) dz
         input(20)=1
         GO TO 7
c ..... Type of redshift step: 
c       0 -> step = Z_STEP; 1 -> step = Z_STEP*(1+z)
      ELSE IF (cname.eq.param(21)) THEN
         READ(line,*) iztyp
         IF (iztyp.ne.0.and.iztyp.ne.1) THEN
            WRITE(*,*) 'ZSTEP_TYPE must be 0 or 1'
            STOP
         END IF
         input(21)=1
         GO TO 7
c ..... Reddening step used to build hypercube
      ELSE IF (cname.eq.param(22)) THEN
         READ(line,*) averr
         input(22)=1
         GO TO 7
c ..... Check the age of the object < universe age (y/n)
      ELSE IF (cname.eq.param(23)) THEN
         cage=line(1:1)
         input(23)=1
         GO TO 7
c ..... Probability threshold for the study of secondary maxima
      ELSE IF (cname.eq.param(24)) THEN
         READ(line,*) pthresh
         input(24)=1
         GO TO 7
c ..... Output SED: 
c       0 -> erg/s/cm^2/A ; 1 -> microJansky ; 2 -> AB magnitudes
      ELSE IF (cname.eq.param(25)) THEN
         READ(line,*) iunit_sed
         IF (iunit_sed.ne.0.and.iunit_sed.ne.1.and.iunit_sed.ne.2) THEN
            WRITE(*,*) 'OUTPUT_TYPE must be 0, 1 or 2'
            STOP
         END IF
         input(25)=1
         GO TO 7
c ..... File .obs_sed (y/n)
      ELSE IF (cname.eq.param(26)) THEN
         csed1=line(1:1)
         input(26)=1
         GO TO 7
c ..... File .temp_sed (y/n)
      ELSE IF (cname.eq.param(27)) THEN
         csed2=line(1:1)
         input(27)=1
         GO TO 7
c ..... File .log_phot (y/n)
      ELSE IF (cname.eq.param(28)) THEN
         clog=line(1:1)
         input(28)=1
         GO TO 7
c ..... File .cat_phot (y/n)
      ELSE IF (cname.eq.param(29)) THEN
         ccat=line(1:1)
         input(29)=1
         GO TO 7
c ..... File .z_phot (y/n)
      ELSE IF (cname.eq.param(30)) THEN
         cz=line(1:1)
         input(30)=1
         GO TO 7
c ..... File .PDZ (y/n)
      ELSE IF (cname.eq.param(42)) THEN
         cPDZ=line(1:1)
         input(42)=1
         GO TO 7
c Optional parameters
c ..... Cluster redshift (optional)
      ELSE IF (cname.eq.param(31)) THEN
         READ(line,*) z_clus
         input(31)=1
         GO TO 7
c ..... Limits for absolute magnitude (optional)
      ELSE IF (cname.eq.param(32)) THEN
         READ(line,*) amabs_1
         input(32)=1
         GO TO 7
      ELSE IF (cname.eq.param(33)) THEN
         READ(line,*) amabs_2
         input(33)=1
         GO TO 7
c ..... Matrix/obj (optional)
      ELSE IF (cname.eq.param(34)) THEN
         cout_m=line(1:1)
         input(34)=1
         GO TO 7
c ..... Spectrum/obj (optional)
      ELSE IF (cname.eq.param(35)) THEN
         cout_s=line(1:1)
         input(35)=1
         GO TO 7
c .... galactic dereddening (optional)
      ELSE IF (cname.eq.param(36)) THEN
         READ(line,*) ebv_mw
         input(36)=1
         GO TO 7
c ..... INDIVIDUAL galactic dereddening (y/n)
      ELSE IF (cname.eq.param(39)) THEN
         READ(line,*) nebv
         input(39)=1
         GO TO 7
c .... z_min for integrated P_z -EDISCS P_int- (optional)
      ELSE IF (cname.eq.param(37)) THEN
         READ(line,*) zp1
         input(37)=1
         GO TO 7
c .... z_max for integrated P_z -EDISCS P_int- (optional)
      ELSE IF (cname.eq.param(38)) THEN
         READ(line,*) zp2
         input(38)=1
         GO TO 7
c ..... File containing "reference" ascii spectra filenames
      ELSE IF (cname.eq.param(40)) THEN
         READ(line,'(A)') filedef_ref
         input(40)=1
         GO TO 7
c ..... Luminosity prior on reference M_ABS
      ELSE IF (cname.eq.param(43)) THEN
         READ(line,*) al_star
         input(43)=1
         GO TO 7
c ..... Luminosity prior on reference ALPHA
      ELSE IF (cname.eq.param(44)) THEN
         READ(line,*) alpha
         input(44)=1
         GO TO 7
      ELSE 
c         WRITE(*,'(a,1x,a)') 'Unknown parameter (ignored)',cname
         GO TO 7
      END IF

 10   CONTINUE
      CLOSE(77)
      DO i=1,30          ! Last parameters are optional
         IF (input(i).eq.0.and.i.ne.12) THEN
            WRITE(*,*) 'Missed parameter ', param(i)
            STOP
         END IF
      END DO
      IF (input(12).eq.0) THEN ! Consistency with the previous version
         r_lyf=1.
         nlyf=1
      END IF

c ..... Control of optional parameters      
      IF (input(31).eq.1.and.icat.eq.0) THEN
         IF (z_clus.lt.zmin.or.z_clus.gt.zmax) THEN
            WRITE(*,*) 'Z_CLUSTER must be in the interval [Z_MIN,Z_MAX]'
            STOP
         END IF
      END IF
      iout_m=1
      iout_s=1
      IF (input(34).eq.1.and.
     .     cout_m.eq.'y'.or.cout_m.eq.'Y') iout_m=2
      IF (input(35).eq.1.and.
     .     cout_s.eq.'y'.or.cout_s.eq.'Y') iout_s=2

      av_mw=3.1*ebv_mw

      IF (law_rd.eq.0) THEN
         avmin=0.0
         avmax=0.0
         averr=1.0
      END IF
      nred=INT(ABS(avmax-avmin)/averr)+1
      IF (nred.gt.mxred) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of reddening steps exceeding the maximum [max=',
     .        mxred,']'
         STOP
      END IF

c...... P_int setup

        iz1=1
        iz2=int((zp2-zp1)/dz)+1
c................................

      OPEN(10,FILE=filedef,STATUS='old',ERR=78)
      iii=1
 2230 READ(10,'(A)',ERR=2220,END=2220) line_sed
      IF (line_sed(1:1).eq.'#'.or.line_sed.eq.' ') GO TO 2230
      last=INDEX(line_sed,' ')-1
      NAMECH(iii)=line_sed(1:last)
      line_sed=line_sed(last+1:)
      DO i=1,80
         IF (line_sed(i:i).ne.' ') THEN
            line_sed=line_sed(i:)
            GO TO 2221
         END IF
      END DO
      WRITE(*,*) 'Missed type of SED file ', NAMECH(iii)
      STOP
 2221 sed_abc(iii)=line_sed(1:2)
      iii=iii+1
      GO TO 2230
 2220 n_files=iii-1
      IF (n_files.gt.mxtyp) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of files exceeding the maximum [max=',mxtyp,']'
         STOP
      END IF
      WRITE(*,*) 'There are ',n_files,' synthetic spectra'
      CLOSE(10)

c......................................
c..... Reference ASCII SED files

      n_files_ref=0

      if(input(40).gt.0)then

      OPEN(10,FILE=filedef_ref,STATUS='old',ERR=78)
      iii=1
22300 READ(10,'(A)',ERR=22200,END=22200) line_sed
      IF (line_sed(1:1).eq.'#'.or.line_sed.eq.' ') GO TO 22300
      last=INDEX(line_sed,' ')-1
      NAMECH_ref(iii)=line_sed(1:last)
      line_sed=line_sed(last+1:)
      DO i=1,80
         IF (line_sed(i:i).ne.' ') THEN
            line_sed=line_sed(i:)
            GO TO 22210
         END IF
      END DO
      WRITE(*,*) 'Missed type of SED file ', NAMECH_ref(iii)
      STOP
22210 sed_abc_ref(iii)=line_sed(1:2)
      iii=iii+1
      GO TO 22300
22200 n_files_ref=iii-1
      IF ((n_files_ref+n_files).gt.mxtyp) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .'Number of reference types exceeding the maximum [max=',
     . mxtyp-n_files,']'
         STOP
      END IF
      WRITE(*,*)'There are ',n_files_ref,' reference spectral types'
      CLOSE(10)

      endif
c......................................

       CALL filter0   !reading FILTER.RES

      OPEN(10,FILE=filt_file,STATUS='old',ERR=80)
      i=0
 134  CONTINUE
      READ(10,'(A)',ERR=135,END=135) line
      IF (line(1:1).eq.'#'.or.line.eq.' ') GO TO 134
      i=i+1
      trans_type(i)=2  !in photons if old version 

c... Test on existence of trans_type(i): 
      ncomp3=0 !number of blanks
      DO k=2,80
      IF (line(k:k).eq.' '.and.line(k-1:k-1).ne.' ')ncomp3=ncomp3+1
      END DO

c... composite filters
      ncomp=1
      DO k=1,80
         IF (line(k:k).eq.'&') THEN
            line(k:k)=' '
            ncomp=ncomp+1
         END IF
      END DO

      IF (ncomp.eq.1) THEN 
         if(ncomp3.gt.5)then
         READ(line,*,ERR=82) nfil(i),c_corr(i),law_nd(i),
     .        mag_nd(i),err_nd(i),trans_type(i)
         else !old file
         READ(line,*,ERR=82) nfil(i),c_corr(i),law_nd(i),
     .        mag_nd(i),err_nd(i)
        endif

        IF (law_nd(i).ne.0.and.law_nd(i).ne.1.and.law_nd(i).ne.2.and.
     .        law_nd(i).ne.3) THEN
            WRITE(*,*) 'Non detection law must be 0, 1, 2 or 3'
            WRITE(*,*) 'Check filter ', nfil(i)
            STOP
         END IF
         IF (nfil(i).gt.icomp) THEN
            WRITE(*,1998) nfil(i),icomp
            STOP
         ELSE
c ..... simple filter

c ... trans_type(i)= 1 : in energy (default = 2)
         IF (trans_type(i).lt.2)then
         iii=nfil(i)
         DO m=1,np(iii)
            r(m,iii,2)=r(m,iii,2)/r(m,iii,1)
         END DO
         endif
c..................
            kfil(1)=nfil(i)
            CALL compos_new(1,kfil,wl(i),surf(i),bp1(i))
            GO TO 134
         END IF
      ELSE 
         if(ncomp3.gt.5)then
         READ(line,*,ERR=82) (kfil(j),j=1,ncomp),c_corr(i),law_nd(i),
     .        mag_nd(i),err_nd(i),trans_type(i)
         else !old file
         READ(line,*,ERR=82) (kfil(j),j=1,ncomp),c_corr(i),law_nd(i),
     .        mag_nd(i),err_nd(i)
        endif

         IF (law_nd(i).ne.0.and.law_nd(i).ne.1.and.law_nd(i).ne.2.and.
     .        law_nd(i).ne.3) THEN
            WRITE(*,*) 'Non detection law must be 0, 1, 2 or 3'
            WRITE(*,*) 'Check filter ', nfil(i)
            STOP
         END IF
c ..... composite filter
c ... trans_type(i)= 1 : in energy (default = 2)
         IF (trans_type(i).lt.2)then
         do j=1,ncomp
         iii=kfil(j)
         DO m=1,np(iii)
            r(m,iii,2)=r(m,iii,2)/r(m,iii,1)
         END DO
         enddo
         endif
c..................
         icomp=icomp+1
         nfil(i)=icomp
         CALL compos_new(ncomp,kfil,wl(i),surf(i),bp1(i))
         GO TO 134
      END IF
 1998 FORMAT('Filter number ',I4, ' out of range!',/,
     .       'Maximum = ',I4)

 135  n_filter=i
      IF (n_filter.gt.mxfil) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of filters exceeding the maximum [max=',mxfil,']'
         STOP
      END IF
      WRITE(*,*) 'There are ',n_filter,' filters'
      nfil(0)=nfilt_mabs(1) ! filter for absolute magnitude estimates -limits-
      kfil(1)=nfil(0)
      CALL compos_new(1,kfil,wl(0),surf(0),bp1(0))
      CLOSE(10)

c.....................................................................
c    Integration of filters for abs. magnitudes
c    the first one remains the "reference" for M_lambda limits 
c.....................................................................

      do i=1,nfmab
         do ii=1,ncomp2(i)
         IF (kfil2(i,ii).gt.icomp) THEN
         WRITE(*,*)'Absolute Mag. filter number id. is ',kfil2(i,ii)
         WRITE(*,*)'The maximum value accepted is ',icomp
         STOP
         endif
         enddo
      enddo

      do i=1,nfmab
       if (ncomp2(i).eq.1)then   !simple filter
            nfilt_mabs(i)=kfil2(i,1)
            kfil(1)=kfil2(i,1)
       CALL compos_new(1,kfil,wl(n_filter+i),
     . surf(n_filter+i),bp1(n_filter+i))
       else                     !composite filter
            icomp=icomp+1
            nfilt_mabs(i)=icomp
            do j=1,ncomp2(i)
            kfil(j)=kfil2(i,j)
            enddo
            CALL compos_new(ncomp2(i),kfil,wl(n_filter+i),
     .      surf(n_filter+i),bp1(n_filter+i))
       END IF
      enddo

c...... we "add" these filters to the list
      do i=1,nfmab
      nfil(n_filter+i)=nfilt_mabs(i)
      enddo
      nfil(0)=nfilt_mabs(1) ! filter for absolute magnitude estimates -limits-
      kfil(1)=nfil(0)
      CALL compos_new(1,kfil,wl(0),surf(0),bp1(0))

      do i=1,n_filter+nfmab
      write(*,*)i,nfil(i),wl(i),surf(i),bp1(i)
      enddo

c.....................................................................
      OPEN(10,FILE=cat_in,STATUS='old',ERR=79)
c ..... Reading the header (thanks to V. Le Brun)
      header = .FALSE.
      READ(10,'(A)',ERR=79) header_line
      IF (header_line(1:1).eq.'#'.or.header_line.eq.' ') THEN
         header= .TRUE.
         i=1
 44      READ(10,'(A)',ERR=79) header_line
         IF (header_line(1:1).eq.'#'.or.header_line.eq.' ') THEN
            i=i+1
            GO TO 44
         ELSE
            n_head=i
         END IF
      END IF
      CLOSE(10)
      OPEN(10,FILE=cat_in,STATUS='old',ERR=79)
c ..... Skipping the header (thanks to V. Le Brun)
      IF (header) THEN
         DO i=1,n_head
            READ(10,'(A)') header_line
         END DO
      END IF

      IF (icat.eq.0) THEN
         iob=0
 1340    CONTINUE
         iob=iob+1
         READ(10,*,ERR=1350,END=1350) id,(amag(i),i=1,n_filter),
     .        (err_p(i),i=1,n_filter)
         z1=zmin
         z2=zmax
         GO TO 1340
      ELSE IF (icat.eq.1) THEN
         iob=0
 1341    CONTINUE
         iob=iob+1
         READ(10,*,ERR=1350,END=1350) id,z1,z2,(amag(i),i=1,n_filter),
     .        (err_p(i),i=1,n_filter)
         IF(z1.lt.zmin) zmin=z1
         IF(z2.gt.zmax) zmax=z2
         GO TO 1341
      ELSE
         GO TO 79
      END IF
 1350 nob=iob-1
      WRITE(*,*) 'There are ',nob,' objects in the catalogue'
      CLOSE(10)

      IF (iout_m.gt.1.and.nob.gt.100) THEN
         iout=0
         WRITE(*,'(a,i6,a)') ' You chose output matrix for all ',
     .        nob, ' objects' 
         WRITE(*,'(a)') ' Are you sure?'
         WRITE(*,*) 'Type 0 to continue with matrix, 1 without matrix'
         WRITE(*,*) '[<return>=1]'
         READ(*,'(a)') line 
         IF (line.ne.' ') THEN
            READ(line,*) iout
            IF (iout.eq.0) iout_m=2
         END IF
      END IF
      z(0)=0.
      z(1)=zmin
      i=1
 88   i=i+1
      IF (iztyp.eq.0) THEN
         z(i)=z(i-1)+dz
      ELSE IF (iztyp.eq.1) THEN
         z(i)=z(i-1)+(1.+z(i-1))*dz
      END IF
      IF (zmax-z(i).gt.1.e-3) GO TO 88
      nz=i

      IF (nz.gt.mxz) THEN
         WRITE(*,'(1x,a,i4,a)') 
     .        'Number of redshift steps exceeding the maximum [max=',
     .        mxz,']'
         WRITE(*,*) 'Choose a larger Z_STEP or a smaller Z_MAX'
         STOP
      END IF

c -----------------------------------------------------------------------
c                       Reading synthetic spectra 
c -----------------------------------------------------------------------

c ..... Vega

      WRITE (*,'(1x,a)') 'Reading reference sed:'
      OPEN(10,FILE=spec_vega,STATUS='old')
      READ(10,'(A)') line
      WRITE(*,'(1x,a)') line (1:78)
      iii=1
 223  READ(10,*,ERR=222,END=222) w_vega(iii),f_vega(iii)
      iii=iii+1
      GO TO 223
 222  iw_vega=iii-1
      CLOSE(10)
      WRITE(*,*) 'There are ',iw_vega,' points in Vega'
      jread=iw_vega

c ..... SED

      WRITE(*,*) 'Reading synthetic spectra :'
      DO i=1,n_files
         WRITE(*,2005) i, NAMECH(i)
c GISSEL's templates
         IF (sed_abc(i).eq.'BC') THEN
            OPEN(1,FILE=NAMECH(i),STATUS='old',
     .           FORM='unformatted',ERR=25)
            READ (1) KS(i),(T_red(ii),ii=1,KS(i)) ! KS = n. temporal intervals
            READ (1) IW(i),(W(ii,i),ii=1,IW(i)) !Number of wavelengths
            DO j=1,KS(i)
               READ (1) IW(i),(F_red(ii,j,i),ii=1,IW(i))
            ENDDO
            CLOSE(1)
c.. starburst99 SEDs templates
          ELSE IF (sed_abc(i).eq.'S') THEN
             OPEN(1,FILE=NAMECH(i),STATUS='old',
     .       FORM='formatted',ERR=25)
             do k=1,3
             read(1,'(A)')line   !3 header lines
             enddo
             ii=1
 3301        READ(1,*,ERR=3401,END=3401) W(ii,i),(F_red(ii,j,i), j=1,36)
             ii=ii+1
             GO TO 3301
 3401        iw(i)=ii-1
             KS(i)=37
             CLOSE(1)
c extended CWW's templates
         ELSE IF (sed_abc(i).eq.'AS'.or.sed_abc(i).eq.'A ')THEN
            OPEN(1,FILE=NAMECH(i),STATUS='old',
     .           FORM='formatted',ERR=25)
            ii=1
 33         READ(1,*,ERR=34,END=34) W(ii,i),F_red(ii,2,i)
            ii=ii+1
            GO TO 33
 34         iw(i)=ii-1
            CLOSE(1)
c Maraston templates
         ELSE IF (sed_abc(i).eq.'MA'.or.sed_abc(i).eq.'ma')THEN
            sed_abc(i)='MA'
            OPEN(1,FILE=NAMECH(i),STATUS='old',
     .           FORM='formatted',ERR=25)
            ii=1
            KS(i)=1
            T_red(1)=0.0
            j=1
            test_age=0.0
 3309        read(1,'(A)',ERR=3409,END=3409)line  
            if(line(1:1).ne.'#')READ(line,*) 
     .      T_red(j),W(ii,i),F_red(ii,j,i)
            if(ii.eq.1)test_age=T_red(j)
            ii=ii+1      ! wavelength
            if(T_red(j).gt.test_age)then  !new age mark
            KS(i)=KS(i)+1
            ii=1
            j=j+1
            endif
            GO TO 3309
 3409        iw(i)=ii-1
c            write(*,*)(t_red(j),j=1,KS(i))
            CLOSE(1)
c..............................
         ELSE
            WRITE(*,*) 'Don''t know how to read :'
            WRITE(*,'(A)') NAMECH(i)
            STOP
         END IF
      END DO

c ..... REFERENCE SEDs

      if(input(40).gt.0)then

      WRITE(*,*) 'Reading REFERENCE ASCII spectra :'
      DO i=1,n_files_ref
         WRITE(*,2005) i, NAMECH_ref(i)
      IF (sed_abc_ref(i).eq.'AS'.or.sed_abc_ref(i).eq.'A ')THEN
            OPEN(1,FILE=NAMECH_ref(i),STATUS='old',
     .           FORM='formatted',ERR=25)
            ii=1
 330   READ(1,*,ERR=340,END=340) W_ref(ii,i),F_ref(ii,2,i)
       ii=ii+1
       GO TO 330
 340   iw_ref(i)=ii-1
       CLOSE(1)
       ELSE
            WRITE(*,*) 'Don''t know how to read :'
            WRITE(*,'(A)') NAMECH_ref(i)
           STOP
         END IF
      END DO

      endif

 2005 FORMAT(' Type ', I2, ' : ', A60)
c -----------------------------------------------------------------------
c                           Rebinning in age
c -----------------------------------------------------------------------

      DO i=1,n_files
         last=INDEX(NAMECH(i),' ')-1
         NAMECH(i)=NAMECH(i)(1:last)
c templates with 221 ages -> 51 after the rebinning
         IF (sed_abc(i).eq.'BC') THEN
            DO k=1,7            ! age=0 to 0.01 Gyr
               j=k*10
               t(k)=t_red(j)
               DO ii=1,iw(i)
                  F(ii,k,i)=F_red(ii,j,i)
                  IF (w(ii,i).lt.912.) F(ii,k,i)=0.
               END DO
            END DO
            DO k=8,34           ! age=0.01 to 3.0 Gyr
               j=70+(k-7)*3
               t(k)=t_red(j)
               DO ii=1,iw(i)
                  F(ii,k,i)=F_red(ii,j,i)
                  IF (w(ii,i).lt.912.) F(ii,k,i)=0.
              END DO
            END DO
            DO k=35,51          ! age=3.0 to 20.0 Gyr
               j=151+(k-34)*4
               t(k)=t_red(j)
               DO ii=1,iw(i)
                  F(ii,k,i)=F_red(ii,j,i)
                  IF (w(ii,i).lt.912.) F(ii,k,i)=0.
               END DO
            END DO
            KS(i)=51
c starburst99 templates
          ELSE IF (sed_abc(i).eq.'S')THEN
          t(1)=t_sb(1)*1.e9
          DO k=1,36
          t(k+1)=t_sb(k+1)*1.e9
          DO ii=1,iw(i)
          F(ii,k+1,i)=0.0
          if(F_red(ii,k,i).gt.-10.0)then
          F_red(ii,k,i)=F_red(ii,k,i)-30.0  !Unites 10**30 erg/cm2
          F(ii,k+1,i)=10**F_red(ii,k,i)
          endif
          IF (w(ii,i).lt.912.) F(ii,k+1,i)=0.
             enddo
             enddo
             KS(i)=37
c Maraston templates
         ELSE IF (sed_abc(i).eq.'MA'.or.sed_abc(i).eq.'ma')THEN
           do k=1,KS(i)
            t(k)=t_red(k)*1.e9
c            write(*,*)k,t(k),t_red(k)
             DO ii=1,iw(i)
             F(ii,k,i)=F_red(ii,K,i)
             IF (w(ii,i).lt.912.) F(ii,k,i)=0.
               END DO
            END DO
c templates with only 1 age 
         ELSE IF (sed_abc(i).eq.'AS'.or.sed_abc(i).eq.'A ')THEN
            DO ii=1,iw(i)
               F(ii,2,i)=F_red(ii,2,i)
               IF (w(ii,i).lt.912.) F(ii,2,i)=0.
            END DO
            KS(i)=2
         END IF
      END DO
c -----------------------------------------------------------------------
c                    Filters, .out_phot and .header_zphot files
c -----------------------------------------------------------------------
      OPEN(13,FILE=file_header,STATUS='unknown')

      OPEN(18,FILE=filterout,STATUS='unknown')
      WRITE(18,'(a)')
     .     '##########################################################'
      WRITE(18,'(a,1x,a)')
     .     '# This is the output file of hyperz - Version '//vv
      WRITE(18,'(a)')
     .     '##########################################################'
      CALL fdate(date)
c      CALL fdate_(date)   ! IBM
      WRITE(18,'(a)') '# Date: '//date
      WRITE(18,'(a)') '# Photometric catalogue : '
      last=INDEX(cat_in,' ')-1
      WRITE(18,'(2x,a)')  cat_in(1:last)
      WRITE(18,'(a)') 
     .     '# Templates SEDs:'
      DO itype=1,n_files
         last=INDEX(NAMECH(itype),' ')-1
         WRITE(18,'(2x,i2,2x,a,2x,a)') itype,NAMECH(itype)(1:last),
     .     sed_abc(itype)
      END DO

      WRITE(18,'(a)') 
     .     '# REFERENCE Templates SEDs:'
      DO itype=1,n_files_ref
         last=INDEX(NAMECH_ref(itype),' ')-1
         WRITE(18,'(2x,i2,2x,a,2x,a)') itype,NAMECH_ref(itype)(1:last),
     .     sed_abc_ref(itype)
      END DO

      WRITE(18,'(a)') '# Characteristics of filters'
      WRITE(18,'(a)') 
     .'#  n     wl_eff     surface   bandpass1  conv_AB mag_lim nd T_l'
      zz=0.

c ..... Conversion from AB to standard magnitudes: conv_mag = AB - Vega
c compute f_Vega(nu)
      DO i=1,iw_vega
c         fnu_vega(i)=f_vega(i)*2.E-17*w_vega(i)*w_vega(i)/c
         fnu_vega(i)=f_vega(i)*2.*w_vega(i)*w_vega(i)/2.9979
         rnu_vega(i)=c/w_vega(i)
      END DO

c      write(*,*)(nfil(k),k=0,n_filter+nfmab)

      DO k=0,n_filter+nfmab
         vega(k)=filter(nfil(k),w_vega,f_vega,iw_vega,zz) ! unit 2.E-17
         zeropoint(k)=-color(nfil(k),nfilt_mabs(1),
     .                       w_vega,f_vega,iw_vega,zz)

c transmission function R(nu): R(nu) d(nu)= R(lambda) d(lambda)
         DO i=1,np(nfil(k))
            wrnu(i)=c/r(i,nfil(k),1) ! frequency
            rrnu(i)=r(i,nfil(k),2)*c/(wrnu(i)*wrnu(i)) ! R(nu)
         END DO
c R(nu)*f_vega(nu)
         DO i=1,np(nfil(k))
            l=0
            ffnu(i)=rrnu(i)*rlin(wrnu(i),rnu_vega,fnu_vega,iw_vega,l)
         END DO
c integrals with trapezoids
         f1=trapz1(wrnu,ffnu,np(nfil(k)))
         f2=trapz1(wrnu,rrnu,np(nfil(k)))
         conv_mag(k)=-2.5*(-35.+LOG10(f1/f2))-48.60
c ..... Wavelength limit for the break in each filter 
c         aw_limit(k)=r(np(nfil(k))-1,nfil(k),1)
      END DO
      DO k=1,n_filter+nfmab
c ..... Flux corresponding to the limiting magnitude (N.B. unit 2.E-17)
         WRITE(18,1999) nfil(k),wl(k),surf(k),bp1(k),conv_mag(k),
     .        mag_nd(k),law_nd(k),trans_type(k)
         IF (magtyp.eq.0) fl_nd(k)=vega(k)*
     .           10**(-0.4*(mag_nd(k)+c_corr(k)))/surf(k) 
         IF(magtyp.eq.1)fl_nd(k)=(10**(-0.4*(mag_nd(k)+c_corr(k)+48.6))*
     .        (2.9979E+18/(wl(k)**2)))/2.E-17  
         mag_nd(k)=-2.5*LOG10(fl_nd(k)*surf(k)/vega(k)) ! Vega limiting mag.
      END DO
 1999 FORMAT(2X,I3,1X,F11.3,1X,2(F10.3,1X),1X,F7.3,1X,F7.3,1X,I2,1X,I2)
      WRITE(18,'(a)') '#    zmin      zmax     zstep'
      WRITE(18,'(4X,3(F5.2,5X))') zmin,zmax,dz
      WRITE(18,'(a)') '#    AVmin     AVmax    AVstep  red_law'
      WRITE(18,'(4X,3(F5.2,5X),I2)') avmin,avmax,averr,law_rd
      WRITE(18,'(a)') 
     .     '#    Lyman_forest [factor(s) multiplying <tau_eff>]'
      IF (nlyf.eq.1) WRITE(18,'(4X,F6.3)') r_lyf
      IF (nlyf.eq.3) WRITE(18,'(4X,3(F6.3,2X))') 
     .     1./r_lyf,r_lyf/r_lyf,r_lyf
      WRITE(18,'(a)') '#    Mmin      Mmax     Mfilter  mag type'
      IF (magtyp.eq.0) WRITE(18,'(4X,2(F7.2,2X),4X,I3,6X,A)') 
     .     amabs_1,amabs_2,nfilt_mabs(1),'Vega'
      IF (magtyp.eq.1) WRITE(18,'(4X,2(F7.2,2X),4X,I3,6X,A)') 
     .     amabs_1,amabs_2,nfilt_mabs(1),'AB'
      WRITE(18,'(a)') '#    Omega_0   Omega_v  H_0'
      WRITE(18,'(4X,3(F5.2,5X))') om,ov,h0
      if(nebv.gt.0)WRITE(18,'(a)')
     .'Applied individual E(B-V) dereddening correction'
      if(ebv_mw.gt.0.0)WRITE(18,*)
     .'General galactic E(B-V) dereddening correction (mags): ',ebv_mw
      if(input(43).gt.0)then
      WRITE(18,'(a)') '#   Luminosity prior on M_ABS '
      WRITE(18,*)'M_* = ',al_star,' Alpha = ',alpha
      else
      WRITE(18,'(a)') '#   No Luminosity prior applied'
      endif

cDS  Save main input file for completeness as *.hyperz (default will be *.hyperz):
       sys_return=INDEX(filein,' ')
       iblank=INDEX(spectra,' ')
       command_name='cp '//filein(1:sys_return-1)//' '//spectra(1:iblank
     & -1)//'.hyperz'
       sys_return = system(command_name)

c -----------------------------------------------------------------------
c                          Building Hypercube! 
c -----------------------------------------------------------------------
      WRITE(*,'(1x,a)') 'Building hypercube ...'
      part_time=cputime()
      DO ired=1,nred
         av=avmin+(ired-1)*averr
         DO i=0,nz
            IF (cage.eq.'y'.or.cage.eq.'Y') THEN
               IF (z(i).le.0.) THEN
                  ageuni=tz(h0,om,ov,0.)
               ELSE
                  ageuni=tz(h0,om,ov,z(i))
               END IF
            END IF
            DO itype=1,n_files
               IF (cage.eq.'y'.or.cage.eq.'Y') THEN
                  iagemax=2
        IF (sed_abc(itype).eq.'BC'.or.sed_abc(itype).eq.'S'.
     .  or.sed_abc(itype).eq.'MA') THEN
c                  IF (sed_abc(itype).eq.'BC') THEN
c DS                  ! if age(Universe) > oldest age of SEDs: allow all SEDs
                     if (ageuni.gt.(T(KS(itype))/1.e9)) then
                        iagemax=KS(itype)
                     else
                    ! otherwise determine age limit
                     DO iage=2,KS(itype)
                        IF (ageuni.gt.(T(iage-1)/1.e9).and.
     .                       ageuni.le.(T(iage)/1.e9)) iagemax=iage
                     END DO
                     endif
                  END IF
               ELSE
                  iagemax=KS(itype)
               END IF
               DO iage=2,iagemax
                  DO k=1,iw(itype)
                     y(k)=f(k,iage,itype)
                     ww(k)=w(k,itype)
                  END DO
                  IF (av.gt.0.0) THEN
                     CALL red_spec(ww,y,av,yob,1,iw(itype),law_rd) 
                     DO k=1,iw(itype)
                        y(k)=yob(k)
                     END DO
                  END IF
c number of filters + the filter for absolute mag estimates
                  DO ilyf=1,nlyf
                     fact_lyf=r_lyf**(ilyf-1)*(1./r_lyf)
                     DO k=1,iw(itype)
                        yob(k)=y(k)
                     END DO
c ..... Lyman alpha forest
                     CALL lyman_forest(ww,yob,iw(itype),z(i))
                     DO k=0,n_filter+nfmab
                        flux(k,i,iage,itype,ired,ilyf)=
     .                       filter(nfil(k),ww,yob,iw(itype),z(i))
     .                       /surf(k)
                     END DO
                  END DO
               END DO
            END DO
         END DO
      END DO

      WRITE(*,'(1x,a)') '                      ... done'
      part_time=cputime()-part_time
      WRITE(*,'(1X,A,F12.3,A)') 'CPU time involved : ',part_time,
     .     ' seconds'

c.......................................................
c... Covolume (dV/dz(z)) between zmin & zmax (for luminosity prior if activated):
c... And default prior == 1.0
        vol(1)=0.0
        prior(1)=1.0
        do i=2,nz
        E=sqrt(om*((1.0+z(i))**3)+((1.0-om-ov)*((1.0+z(i))**2))+ov)
        vol(i)=(10.0**(2.0*0.2*(d_mod(h0,om,ov,z(i))+5.0)))*1.e-12*
     .  (2.9979E+5/h0)/
     .  (((1.0+z(i))**2)*E) !Mpc-2
        prior(i)=1.0
        enddo
c.......................................................


c      write(*,*)nz,nlyf,n_filter,n_filter+nfmab,nred,n_files
c----------------------------------------------------------------------
c   Reference templates hypercube
c----------------------------------------------------------------------
      if(n_files_ref.gt.0)then
      WRITE(*,'(1x,a)') 'Building Reference SED hypercube ...'

c templates with only 1 age 
            DO ii=1,iw_ref(i)
               IF (w_ref(ii,i).lt.912.) F_ref(ii,2,i)=0.
            END DO

         DO i=0,nz                       !redshift
            DO itype=1,n_files_ref       !reference types
                 DO k=1,iw_ref(itype)
                     y(k)=f_ref(k,2,itype)
                     ww(k)=w_ref(k,itype)
                  END DO
c number of filters + the filter for absolute mag estimates
                  DO ilyf=1,nlyf
                     fact_lyf=r_lyf**(ilyf-1)*(1./r_lyf)
                     DO k=1,iw_ref(itype)
                        yob(k)=y(k)
                     END DO
c ..... Lyman alpha forest
                     CALL lyman_forest(ww,yob,iw_ref(itype),z(i))
                     DO k=0,n_filter+nfmab
                        flux_ref(k,i,itype,ilyf)=
     .                  filter(nfil(k),ww,yob,iw_ref(itype),z(i))
     .                  /surf(k)
                     END DO ! filters

                  END DO   ! Ly forest
               END DO   ! type
            END DO   ! redshift

      WRITE(*,'(1x,a)') '                      ... done'
      endif
c -----------------------------------------------------------------------
c                          Loop over the objects 
c -----------------------------------------------------------------------
      WRITE(*,'(1x,a)') 'I''m working ...'

      IF (ccat.eq.'y'.or.ccat.eq.'Y') 
     .     OPEN(10,FILE=spectra1,STATUS='unknown',ERR=81)
      IF (clog.eq.'y'.or.clog.eq.'Y')
     .     OPEN(14,FILE=spectra2,STATUS='unknown',ERR=81)
      IF (csed1.eq.'y'.or.csed1.eq.'Y')
     .     OPEN(15,FILE=spectra3,STATUS='unknown',ERR=81)
      IF (cz.eq.'y'.or.cz.eq.'Y')
     .     OPEN(17,FILE=spectra4,STATUS='unknown',ERR=81)
      IF (csed2.eq.'y'.or.csed2.eq.'Y')
     .     OPEN(19,FILE=spectra5,STATUS='unknown',ERR=81)
      IF (input(37).eq.1.and.input(38).eq.1)
     .     OPEN(20,FILE=spectra6,STATUS='unknown',ERR=81)  !proba_int
      IF (cPDZ.eq.'y'.or.cPDZ.eq.'Y')
     .     OPEN(21,FILE=file_PDZ,STATUS='unknown',ERR=81)  !PDZ file

      WRITE(fmt1,2006) n_filter,n_filter
      WRITE(fmt2,2007) n_filter,n_filter
      WRITE(fmt3,2010) n_filter
      WRITE(fmt4,2011) n_filter
      WRITE(fmt6,2020) iz2
      WRITE(fmtPDZ,2021)nz 
      if(n_files_ref.gt.0)then
      WRITE(fmt2008,20080)nfmab,n_files_ref
      WRITE(fmt2009,20090)nfmab,n_files_ref
      else
      WRITE(fmt2008,2008)nfmab
      WRITE(fmt2009,2009)nfmab
      endif
      WRITE(fmt2001,2001)nfmab

 2006 FORMAT('(1x,i5,',i2,'(1x,F8.4),',i2,'(1x,F8.4))')
 2007 FORMAT('(1x,i5,',i2,'(1x,E10.3),',i2,'(1x,E10.3))')
 2010 FORMAT('(1x,i5,',i2,'(1x,F8.4))')
 2011 FORMAT('(1x,i5,',i2,'(1x,E10.3))')
 2020 FORMAT('(i10,1x,2(E9.3,1x),',i2,'(E9.3,1x))')
 2021 FORMAT('(i10,1x,',i3,'(E9.3,1x))')

 2001 FORMAT('(i10,1x,f6.3,1x,f8.3,1x,f6.2,1x,i2,1x,i3,1x,f8.4,1x,
     .f5.2,1x,e9.3,1x,6(f6.3,1x),f6.3,1x,f6.2,1x,',
     .i1,'(f7.2,1x),f6.2,1x,F6.3,1x,i3,1x,e11.4)')

2008  FORMAT('(i10,1x,f8.5,1x,f8.3,1x,f6.2,1x,i2,1x,i3,1x,f8.4,1x,
     .f5.2,1x,e9.3,1x,6(f6.3,1x),f8.5,1x,f6.2,1x,',
     .i1,'(f7.2,1x),f6.2,1x,F6.3,1x,
     .f6.3,1x,f6.2,1x,e11.4)')
2009  FORMAT('(i10,1x,f8.5,1x,f8.3,1x,f6.2,1x,i2,1x,i3,1x,f8.4,1x,
     .f5.2,1x,e9.3,1x,6(f6.3,1x),f8.5,1x,f6.2,1x,',i1,
     .'(f7.2,1x),f6.2,1x,f6.3,1x,
     .f6.3,1x,f6.2,1x,F7.2,1x,f6.2,1x,f6.2,2(1x,f8.4),1x,e11.4)')

20080 FORMAT('(i10,1x,f8.5,1x,f8.3,1x,f6.2,1x,i2,1x,i3,1x,f8.4,1x,
     .f5.2,1x,e9.3,1x,6(f6.3,1x),f8.5,1x,f6.2,1x,',
     .i1,'(f7.2,1x),f6.2,1x,F6.3,1x,
     .f6.3,1x,f6.2,1x,i1,1x,',i1,'(f7.2,1x),1x,e11.4)')
20090 FORMAT('(i10,1x,f8.5,1x,f8.3,1x,f6.2,1x,i2,1x,i3,1x,f8.4,1x,
     .f5.2,1x,e9.3,1x,6(f6.3,1x),f8.5,1x,f6.2,1x,',
     .i1,'(f7.2,1x),f6.2,1x,f6.3,1x,
     .f6.3,1x,f6.2,1x,F7.2,1x,f6.2,1x,f6.2,2(1x,f8.4),1x,i1,1x,',
     .i1,'(f7.2,1x),1x,e11.4)')

c      write(*,'(A)')fmt2009
c      write(*,'(A)')fmt2001
c      write(*,'(A)')fmt2008

      OPEN(16,FILE=cat_in,STATUS='old',ERR=79)
c ..... Skipping the header (thanks to V. Le Brun)
      IF (header) THEN
         DO i=1,n_head
            READ(16,'(A)') header_line
         END DO
      END IF

      iob=0
c..............
13400 CONTINUE
c..............

      iob=iob+1
      ebv_in=0.0

      if(nebv.eq.0)then
      IF (icat.eq.0) THEN
         READ(16,*,ERR=13500,END=13500) id,(amag(i),i=1,n_filter),
     .        (err_p(i),i=1,n_filter)
      ELSE IF (icat.eq.1) THEN
         READ(16,*,ERR=13500,END=13500) id,z1,z2,(amag(i),i=1,n_filter),
     .        (err_p(i),i=1,n_filter)
      END IF

      else
      IF (icat.eq.0) THEN
         READ(16,*,ERR=13500,END=13500) id,(amag(i),i=1,n_filter),
     .        (err_p(i),i=1,n_filter),ebv_in
      ELSE IF (icat.eq.1) THEN
         READ(16,*,ERR=13500,END=13500) id,z1,z2,(amag(i),i=1,n_filter),
     .        (err_p(i),i=1,n_filter),ebv_in
      END IF

      endif

      renor_log=0.0    !normalized P(z) for Ediscs/z_cluster calculations
     
      DO iii=1,n_filter
         IF (amag(iii).lt.90.0.and.amag(iii).gt.1.) GO TO 13401
      END DO                  

      WRITE(*,*) 'WARNING: non detected object id ', id
c ..... Write outputs in any case (except .m and .spe)
c ..... File .z_phot: at 13131 : anomalous objects
      zphot(1)=99.

c ..... File .obs_sed
      IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
      DO i=1,n_filter
      fl1(i)=-1.
      err_fl(i)=0.
      amag(i)=99.
      err_p(i)=99.
      END DO
      IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
      WRITE(15,fmt2) id,(fl1(i),i=1,n_filter),
     .(err_fl(i),i=1,n_filter)
      ELSE IF (iunit_sed.eq.2) THEN
      WRITE(15,fmt1) id,(amag(i),i=1,n_filter),
     .(err_p(i),i=1,n_filter)
      END IF
      END IF

      GO TO 13131
         
c ..... Computing the fluxes (unit 2.E-17) and errors in the filters
13401 iw_ob=0
      DO iii=1,n_filter
         i_lyman(iii)=0
         IF (err_p(iii).lt.err_min) err_p(iii)=err_min
         IF (amag(iii).gt.1..and.amag(iii).lt.90.) THEN
            iw_ob=iw_ob+1
            IF (magtyp.eq.0) fl(iii)=vega(iii)*
     .           10**(-0.4*(amag(iii)+c_corr(iii)))/surf(iii) 
        IF(magtyp.eq.1)fl(iii)=(10**(-0.4*(amag(iii)+c_corr(iii)+48.6))*
     .           (2.9979E+18/(wl(iii)**2)))/2.E-17 
            err_fl(iii)=(2.3/2.5)*fl(iii)*err_p(iii)

c ..... Galactic Dereddening (general)
            IF (ebv_mw.ne.0.) THEN
               CALL red_spec(wl(iii),fl(iii),av_mw,fl_ext,2,1,1)
               fl(iii)=fl_ext
            END IF

c ..... Galactic Dereddening (individual)
c       This correction is added on the top of general ebv_mw
            IF (ebv_in.gt.0..and.ebv_in.lt.5.) THEN
               aw_in=3.1*ebv_in  !(-> Av for general MW law)
               CALL red_spec(wl(iii),fl(iii),aw_in,fl_ext,2,1,1)
               fl(iii)=fl_ext
             ELSE IF (ebv_in.lt.0..or.abs(ebv_in).ge.5.) THEN
             write(*,*)'WARNING: E(B-V) = ',ebv_in,' for object ',id
             write(*,*)'Individual E(B-V) ignored - Check input file'
             ebv_in=0.0
            END IF

c amag -> Vega magnitude
            IF (magtyp.eq.1) amag(iii)=amag(iii)-conv_mag(iii)

c            amag(iii)=-2.5*LOG10(fl(iii)*surf(iii)/vega(iii))
         ELSE IF (amag(iii).le.1.) THEN ! out of file objects
            fl(iii)=-1.0
         ELSE IF (amag(iii).ge.90.) THEN ! non detected objects
            IF (law_nd(iii).eq.0) THEN
               fl(iii)=-1.0
               err_fl(iii)=0.
               err_p(iii)=0.
            ELSE IF (law_nd(iii).eq.1) THEN
               fl(iii)=0.
               err_fl(iii)=fl_nd(iii)
         err_p(iii)=(2.5/2.3)*err_fl(iii)/fl_nd(iii)
c               err_p(iii)=err_nd(iii)
               i_lyman(iii)=2
               iw_ob=iw_ob+1
            ELSE IF (law_nd(iii).eq.2) THEN
               fl(iii)=fl_nd(iii)/2.
               err_fl(iii)=fl(iii)
         err_p(iii)=(2.5/2.3)*err_fl(iii)/fl(iii)
c               err_p(iii)=err_nd(iii)
               i_lyman(iii)=1
               iw_ob=iw_ob+1
            ELSE IF (law_nd(iii).eq.3) THEN
               fl(iii)=fl_nd(iii)
               err_p(iii)=err_nd(iii)
               err_fl(iii)=(2.3/2.5)*fl(iii)*err_p(iii)
               i_lyman(iii)=1
               iw_ob=iw_ob+1
            END IF
         END IF
      END DO

      ndeg=iw_ob-1  ! degrees of freedom
      deg=REAL(ndeg)

      IF (ndeg.lt.1) THEN
         WRITE(*,*) 'WARNING: impossible to compute object ', iob

c ..... Write outputs in any case (except .m and .spe)
c ..... File .z_phot at 13131: anomalous objects

         zphot(1)=99.

c ..... File .obs_sed
      IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
      DO i=1,n_filter
      fl1(i)=-1.
      err_fl(i)=0.
      amag(i)=99.
      err_p(i)=99.
      END DO
      IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
      WRITE(15,fmt2) id,(fl1(i),i=1,n_filter),
     .(err_fl(i),i=1,n_filter)
      ELSE IF (iunit_sed.eq.2) THEN
      WRITE(15,fmt1) id,(amag(i),i=1,n_filter),
     .(err_p(i),i=1,n_filter)
      END IF
      END IF

         GO TO 13131
      END IF
c -----------------------------------------------------------------------
c                         Computation 
c -----------------------------------------------------------------------

c ..... Open file for the matrix of results (if case)

      matrix=0
      WRITE(root,'(i8)') id
      DO k=1,8
         IF (root(k:k).ne.' ') THEN
            root=root(k:)
            GO TO 22
         END IF
      END DO
 22   iroot=INDEX(root,' ')-1
      IF (iout_m.gt.1) THEN
         amatrix=root(1:iroot)//'.m'
         matrix=1
         OPEN(11,FILE=amatrix,STATUS='unknown')
      END IF

      IF (icat.eq.1) THEN
         DO i=1,nz-1
            IF (z1.ge.z(i).and.z1.lt.z(i+1)) i1=i
            IF (z2.gt.z(i).and.z2.le.z(i+1)) i2=i+1
         END DO
         IF (z2.gt.z(nz)) i2=nz
      ELSE IF (icat.eq.0) THEN
         i1=1
         i2=nz
      END IF

      IF (z_clus.ge.0.) THEN
      iz_clus=1
      iz_clus05a=1
      iz_clus05b=1
      iz_clus10a=1
      iz_clus10b=1

         DO i=1,nz-1
            IF (z_clus.ge.z(i).and.z_clus.lt.z(i+1)) iz_clus=i
         if(z_clus-0.05.ge.z(i).and.z_clus-0.05.lt.z(i+1))
     .   iz_clus05a=i
         if(z_clus+0.05.ge.z(i).and.z_clus+0.05.lt.z(i+1))
     .   iz_clus05b=i+1
         if(z_clus-0.10.ge.z(i).and.z_clus-0.10.lt.z(i+1))
     .   iz_clus10a=i
         if(z_clus+0.10.ge.z(i).and.z_clus+0.10.lt.z(i+1))
     .   iz_clus10b=i+1
         END DO
         IF (z_clus.eq.z(nz)) iz_clus=nz
      END IF

c... Filter number for absolute magnitude if in the cluster

         dif_w_min=1.e6
         n_fil_wa=1
         DO k=1,n_filter
            dif_w=ABS(wl(k)-wl(0)*(z_clus+1.))
            IF (amag(k).gt.1..and.amag(k).lt.90.) THEN
               IF (dif_w.lt.dif_w_min) THEN 
                  dif_w_min=dif_w
                  n_fil_wa=k
               END IF
            END IF
         END DO

c.... Setup of normalization for Lum prior if needed

      count=0.0
      if(input(43).gt.0)prior(1)=0.0

c -----------------------------------------------------------------------
      DO i=i1,i2                                 !Scan in z
c -----------------------------------------------------------------------

c.... Initializing variables:
         z_min_array(i)=1.E+30
         i_typ(i)=1
         i_age(i)=2
         i_red(i)=1
         i_lyf(i)=1
                        itypea=1
                        iagea=2
                        ireda=1
                        ilyfa=1

         IF (cage.eq.'y'.or.cage.eq.'Y') THEN
            IF (z(i).le.0.) THEN
               ageuni=tz(h0,om,ov,0.)
            ELSE
               ageuni=tz(h0,om,ov,z(i))
            END IF
         END IF

c... Filter number for REFERENCE absolute magnitude at z

        dif_w_min=1.e6
         n_fil_w(i)=1
         DO k=1,n_filter
            dif_w=ABS(wl(k)-wl(0)*(z(i)+1.))
            IF (amag(k).gt.1..and.amag(k).lt.90.
     .           .and.i_lyman(k).lt.1) THEN
               IF (dif_w.lt.dif_w_min) THEN
                  dif_w_min=dif_w
                  n_fil_w(i)=k
               END IF
            END IF
         END DO       
           
c... Filter number for all other absolute magnitudes at z 

         dif_w_min=1.e6
         do ii=1,nfmab
         n_fil_abs(ii,i)=1
         enddo
         do ii=1,nfmab
         dif_w_min=1.e6
         DO k=1,n_filter
         dif_w=ABS(wl(k)-wl(n_filter+ii)*(z(i)+1.))
            IF (amag(k).gt.1..and.amag(k).lt.90.
     .           .and.i_lyman(k).lt.1) THEN
               IF (dif_w.lt.dif_w_min) THEN
                  dif_w_min=dif_w
                  n_fil_abs(ii,i)=k
               END IF
            END IF
         END DO       
c         write(*,*)ii,z(i),n_fil_abs(ii,i)
         enddo
c -----------------------------------------------------------------------
         DO itype=1,n_files     !Scan types
            IF (cage.eq.'y'.or.cage.eq.'Y') THEN
               iagemax=2
         IF (sed_abc(itype).eq.'BC'.or.sed_abc(itype).eq.'S'
     .   .or.sed_abc(itype).eq.'MA') THEN
c DS                  ! if age(Universe) > oldest age of SEDs: allow all SEDs
                     if (ageuni.gt.(T(KS(itype))/1.e9)) then
                        iagemax=KS(itype)
                     else
                     ! otherwise determine age limit
                  DO iage=2,KS(itype)
                     IF (ageuni.gt.(T(iage-1)/1.e9).and.
     .                   ageuni.le.(T(iage)/1.e9)) iagemax=iage
                  END DO
                     endif
               END IF
            ELSE
               iagemax=KS(itype)
            END IF
c -----------------------------------------------------------------------
            DO iage=2,iagemax !Scan ages
c -----------------------------------------------------------------------
               DO ired=1,nred   !Scan reddening
c -----------------------------------------------------------------------
                  DO ilyf=1,nlyf !Scan Lyman Forest
                     fact_lyf=r_lyf**(ilyf-1)*(1./r_lyf)
c -----------------------------------------------------------------------
                     av=avmin+(ired-1)*averr
                     sxx=0.D0
                     sxy=0.D0
                     b=0.D0

c ..... Probability for z(i), iage, itype, ired, ilyf

                     DO k=1,n_filter
                        IF (fl(k).ge.0.) THEN ! no oof objects
                           IF (flux(k,i,iage,itype,ired,ilyf).eq.0..and.
     .                          i_lyman(k).eq.1) THEN
                              fl1(k)=0.
                           ELSE
                              fl1(k)=fl(k)
                              wt=1.D0/(err_fl(k)**2)
                              sxx=sxx+
     .                             DBLE(flux(k,i,iage,itype,ired,ilyf))
     .                             *DBLE(flux(k,i,iage,itype,ired,ilyf))
     .                             *wt
                              sxy=sxy+DBLE(fl1(k))*wt*
     .                             DBLE(flux(k,i,iage,itype,ired,ilyf))
                           END IF
                        ELSE
                           fl1(k)=-1.
                        END IF
                     END DO
                     b=sxy/sxx
                     chi2=0.
                     DO k=1,n_filter
                        IF (fl1(k).ge.0.) THEN ! no oof obj.
                           chi2=chi2+((fl1(k)
     .                          -b*flux(k,i,iage,itype,ired,ilyf))
     .                          /err_fl(k))**2
                        END IF
                     END DO
                     IF (fact_lyf.eq.1..and. ! average Ly for. preferred value
     .                    ABS(chi2-z_min_array(i)).lt.err) THEN
                        z_min_array(i)=chi2
                        bb_array(i)=b
                        i_typ(i)=itype
                        i_age(i)=iage
                        i_red(i)=ired
                        i_lyf(i)=ilyf
                     ELSE IF ((chi2-z_min_array(i)).lt.0.) THEN
                        z_min_array(i)=chi2
                        bb_array(i)=b
                        i_typ(i)=itype
                        i_age(i)=iage
                        i_red(i)=ired
                        i_lyf(i)=ilyf
                     END IF

         IF (sed_abc(itype).eq.'BC'.or.sed_abc(itype).eq.'S'
     .   .or.sed_abc(itype).eq.'MA') THEN
                        age=T(iage)/1.e9
                     ELSE IF (sed_abc(itype).eq.'AS'.or.
     .                       sed_abc(itype).eq.'A ') THEN
                        age=19.9999
                     END IF

                     IF (matrix.gt.0) THEN
                        r_chi2=chi2/deg ! reduced chi2
                b_scale=dlog10(b*(2.0D-17)*4.D0*DBLE(pi))+
     .          (2.D0*dlog10(DBLE(d_lum(h0,om,ov,z(i)))))
            ppp=100.*qchi2(0.5*deg,0.5*r_chi2)
                        WRITE(11,2012) id,z(i),itype,iage,age,av,
     .                       r_chi2,ppp,fact_lyf,b_scale
                     END IF
                  END DO
               END DO
            END DO
         END DO

         q_pro(i)=100.*qchi2(0.5*deg,0.5*z_min_array(i))
         
       IF (sed_abc(i_typ(i)).eq.'BC'.or.sed_abc(i_typ(i)).eq.'S'
     .   .or.sed_abc(i_typ(i)).eq.'MA') THEN
            age=T(i_age(i))/1.e9
         ELSE IF (sed_abc(i_typ(i)).eq.'AS'.or.
     .           sed_abc(i_typ(i)).eq.'A ') THEN
            age=19.9999
         END IF
         
         r_chi2=z_min_array(i)/deg ! reduced chi2
         proba_z(i)=pro_z(deg,z_min_array(i))

         fact_lyf=r_lyf**(i_lyf(i)-1)*(1./r_lyf)

c..............Absolute magnitude ..............................

      if(mscale.lt.1.5)then
c Computing absolute REFERENCE magnitude (z)
c    Closest filter where the object is detected

      abmag(i)=0.0
      bmag=1000.0
      sauce=flux(n_fil_w(i),i,i_age(i),i_typ(i),i_red(i),i_lyf(i))
     .     /flux(n_fil_w(i),0,i_age(i),i_typ(i),i_red(i),i_lyf(i))
      IF (sauce.gt.0.0) bmag=amag(n_fil_w(i))+ 
     .     2.5*LOG10(sauce)-d_mod(h0,om,ov,z(i))
      sauce=flux(0,0,i_age(i),i_typ(i),i_red(i),i_lyf(i))*surf(0)
       IF (sauce.gt.0.0) abmag(i)=bmag-(zeropoint(n_fil_w(i))-
     .     2.5*LOG10(flux(n_fil_w(i),0,i_age(i),
     .     i_typ(i),i_red(i),i_lyf(i))*surf(n_fil_w(i))/sauce))

      IF (magtyp.eq.1) abmag(i)=abmag(i)+conv_mag(0)                                       
      IF (abmag(i).ge.amabs_1.and.abmag(i).le.amabs_2)
     .renor_log=renor_log+proba_z(i)    !normalization on possible zs

c.............................................
c Computing all other absolute magnitudes (z)
c    Closest filter where the object is detected

      do ii=1,nfmab
      abmag2(ii,i)=0.0
      enddo
      zz=0.0

      do ii=1,nfmab
      bmag=1000.0
      sauce=flux(n_fil_abs(ii,i),i,i_age(i),i_typ(i),i_red(i),i_lyf(i))
     ./flux(n_fil_abs(ii,i),0,i_age(i),i_typ(i),i_red(i),i_lyf(i))
      IF (sauce.gt.0.0) bmag=amag(n_fil_abs(ii,i))+ 
     .     2.5*LOG10(sauce)-d_mod(h0,om,ov,z(i))
      sauce=flux(n_filter+ii,0,i_age(i),i_typ(i),i_red(i),
     .i_lyf(i))*surf(n_filter+ii)
c      IF (sauce.gt.0.0) abmag2(ii,i)=bmag-(zeropoint(n_fil_abs(ii,i))-  !! corrected bug v<9
      IF (sauce.gt.0.0) abmag2(ii,i)=bmag+color(nfil(n_fil_abs(ii,i)),
     .nfilt_mabs(ii),w_vega,f_vega,iw_vega,zz)+
     .2.5*LOG10(flux(n_fil_abs(ii,i),0,i_age(i),
     .i_typ(i),i_red(i),i_lyf(i))*surf(n_fil_abs(ii,i))/sauce)

      IF (magtyp.eq.1) abmag2(ii,i)=abmag2(ii,i)+conv_mag(n_filter+ii)     

      enddo


      else   !mscale=2 => b * 2x10^(-17) * F_lambda(model)

c Computing absolute REFERENCE magnitude (z)
c    Closest filter where the object is detected

      abmag(i)=0.0
      bmag=1000.0
      zz=0.0

      sauce=filter(nfil(0),w_vega,f_vega,iw_vega,zz)/(surf(0)*
     .      flux(0,0,i_age(i),i_typ(i),i_red(i),i_lyf(i)))
      IF (sauce.gt.0.0)abmag(i)=2.5*LOG10(sauce)-d_mod(h0,om,ov,z(i))
     .    -2.5*LOG10(bb_array(i))

      IF (magtyp.eq.1) abmag(i)=abmag(i)+conv_mag(0)                                       

      IF (abmag(i).ge.amabs_1.and.abmag(i).le.amabs_2)
     .renor_log=renor_log+proba_z(i)    !normalization on possible zs

c.............................................
c Computing all other absolute magnitudes (z)
c    Closest filter where the object is detected

      do ii=1,nfmab
      abmag2(ii,i)=0.0
      enddo
      zz=0.0

      do ii=1,nfmab
      bmag=1000.0
      sauce=filter(nfil(n_filter+ii),w_vega,f_vega,iw_vega,zz)/
     .(surf(n_filter+ii)*
     .flux(n_filter+ii,0,i_age(i),i_typ(i),i_red(i),i_lyf(i)))
      IF (sauce.gt.0.0)abmag2(ii,i)=2.5*LOG10(sauce)-
     .                 d_mod(h0,om,ov,z(i))-2.5*LOG10(bb_array(i))

      IF (magtyp.eq.1) abmag2(ii,i)=abmag2(ii,i)+conv_mag(n_filter+ii)     

      enddo

      endif

c .........abmag(i)............. done!

c ..... luminosity prior (optional)

      if(input(43).gt.0)then

c... dA²(m,z)=Phi(M)((1+z)**3)(dV/dz)dmdz (number of galaxies 
c... within (z, z+dz) per steradian and magnitude bin (arbitrary normalization)

             afl=(10.0**(-0.4*(abmag(i)-al_star)*(alpha+1)))*
     .         exp(-(10.0**(-0.4*(abmag(i)-al_star))))          !LF shape in Mags
       prior(i)=vol(i)*afl*((1.0+z(i))**3)  ! this is an almost "flat" function (slowly decreassing)
       if(count.lt.prior(i))count=prior(i)  !count=normalisation to 1 for the maximum  P(z)
       endif
c       write(*,*)i,z(i),abmag(i),vol(i),afl,prior(i)
c       read(*,'(A)')line

c.................. END Luminosity prior      


c...................................................................
c...................................................................
      END DO
      CLOSE(11)                 !close output matrix & z loop
c...................................................................
c...................................................................

c ..... Normalisation of luminosity prior (optional)
      if(input(43).gt.0.and.count.gt.0.)then
           do i=i1,i2
           prior(i)=prior(i)/count  !normalisation to 1 for the maximum  P(z)
           enddo
           prior(i1)=0.0
           else      !no Luminosity prior applied 
           do i=i1,i2
           prior(i)=1.0             !normalisation to 1 for the maximum  P(z)
           enddo
           prior(i1)=0.0
         endif

c............Best fit at cluster z (if cluster option is activated):
         IF (z_clus.ge.0.) THEN
           do i=i1,i2
           IF (i.eq.iz_clus) THEN
           itypea=i_typ(i)
           iagea=i_age(i)
           ireda=i_red(i)
           ilyfa=i_lyf(i)
           END IF
           enddo
         endif
c..................................

c...... .log special pour EDISCS

         IF(clog.eq.'y'.or.clog.eq.'Y') THEN
         do i=i1,i2
         r_chi2=z_min_array(i)/deg ! reduced chi2

         b_scale=dlog10(DBLE(bb_array(i))*(2.0D-17)*4.D0*
     .   DBLE(pi))+
     .   (2.D0*dlog10(DBLE(d_lum(h0,om,ov,z(i)))))

         proba=0.0         
      IF (abmag(i).ge.amabs_1.and.abmag(i).le.amabs_2.
     .and.renor_log.gt.0.)proba=pro_z(deg,z_min_array(i))/renor_log ! solution ok

      iagem=i_age(i)
      iredm=i_red(i)
      av=avmin+(iredm-1)*averr
      IF (sed_abc(i_typ(i)).eq.'BC'.or.sed_abc(i_typ(i)).eq.'S'
     .   .or.sed_abc(i_typ(i)).eq.'MA') THEN
         age=T(i_age(i))/1.e9
      ELSE IF (sed_abc(i_typ(i)).eq.'AS'.or.
     .        sed_abc(i_typ(i)).eq.'A ') THEN
         age=19.9999
      END IF
 
      WRITE(14,2030)id,z(i),r_chi2,proba,abmag(i),i_typ(i),i_age(i),
     .age,av,b_scale,prior(i),prior(i)*proba
         enddo
         END IF
 2030 format(i10,1x,f6.3,1x,e11.4,1x,f10.5,1x,f7.2,2(1x,i3),
     .2(1x,f10.5),1x,3(e11.4,1x))
c -----------------------------------------------------------------------
c                     End scan
c -----------------------------------------------------------------------
c ..... Searching minima of chi2

      n_sol=0
      ntop1=0
      ntop2=0

c...... Optional: P(z) corrected by prior(z) in luminosity (computed above):
c       Arbitrary normalization, but we respect the "absolute" xi2 for the best solution

       if(input(43).gt.0)then
       count_max=1.E+30
       DO i=i1,i2
       if(z_min_array(i).lt.count_max)then
       count_max=z_min_array(i)
       endif
       END DO
       count_max=pro_z(deg,count_max)  !Max proba for the first minimum (at least one is OK)

       if(count_max.gt.0.0)then    !we apply the luminosity prior if at least one OK
       DO i=i1,i2
       E=prior(i)*pro_z(deg,z_min_array(i))  !combined proba with normalized prior
       if(E.gt.0.0)then
       z_min_array(i)=2.0*abs(log(E))
       else
       z_min_array(i)=100.0  ! arbitrarily high value (as in pro_z function)
       endif
       END DO
       endif
       endif
c......
c -----------------------------------------------------------------------

      DO i=i1,i2
         dd=1.0
         dd2=1.0
         IF (i.eq.i1) THEN
            dd=z_min_array(i1)-z_min_array(i1+1)
            d1=z_min_array(i1)-z_min_array(i1+1)
            d2=-d1
         ELSE IF (i.eq.i2) THEN
            dd=z_min_array(i2)-z_min_array(i2-1)
            d1=z_min_array(i2)-z_min_array(i2-1)
            d2=-d1
         ELSE
            d1=z_min_array(i)-z_min_array(i-1)
            d2=z_min_array(i+1)-z_min_array(i)
            IF (d1.ne.0.) dd=d2/d1
            IF (d2.ne.0.) dd2=d1/d2
         END IF
c  OLD:IF (dd.le.0..and.d1.lt.0.) THEN
         IF (dd.le.0..and.d1.lt.0..and.d2.gt.0.) THEN ! dd<0 extrema; situation 1 (single minimum)
            n_sol=n_sol+1
            izphot(n_sol)=i
c Situation 2 ("flat" minimum: low resolution in z arround minimum)
         ELSE IF(dd.le.0..and.d1.lt.0..and.d2.le.0..and.ntop1.lt.1.and.
     .   i.gt.i1) THEN ! top1: d2=0, d1<0
         ntop1=i
         ELSE IF(dd2.le.0..and.d1.le.0..and.d2.gt.0..and.ntop1.gt.0)THEN ! top2: d2>0, d1=0
         ntop2=i
         n_sol=n_sol+1
         izphot(n_sol)=ntop1+int((ntop2-ntop1)/2.0)
         ntop1=0
         END IF
      END DO

c ..... Refinement around minima
      DO i=1,n_sol
         izsol=izphot(i)
         IF (izsol.eq.i1) THEN
            dz_f=(z(izsol+1)-z(izsol))/10.
            zmax_f=z(izsol+1)
            zmin_f=z(izsol)
         ELSE IF (izsol.eq.i2) THEN
            dz_f=(z(izsol)-z(izsol-1))/10.
            zmax_f=z(izsol)
            zmin_f=z(izsol-1)
         ELSE
            dz_f=(z(izsol+1)-z(izsol-1))/20.
            zmax_f=z(izsol+1)
            zmin_f=z(izsol-1)
         END IF
         IF (zmin_f.lt.z(i1)) zmin_f=z(i1)
         IF (zmax_f.gt.z(i2)) zmax_f=z(i2)
         av=avmin+(i_red(izsol)-1)*averr
         fact_lyf=r_lyf**(i_lyf(izsol)-1)*(1./r_lyf)
         CALL zfit_min(i_age(izsol),i_typ(izsol),av,
     .        zmin_f,zmax_f,dz_f,chi2,zop)
         IF (izsol.eq.i1.or.izsol.eq.i2) THEN 
            IF (chi2.lt.z_min_array(izsol)) THEN
               zphot(i)=zop
               chi2sol(i)=chi2
               bb(i)=bm
            ELSE 
               zphot(i)=z(izsol)
               chi2sol(i)=z_min_array(izsol)
               bb(i)=bb_array(izsol)
            END IF
         ELSE
            zphot(i)=zop
            chi2sol(i)=chi2
            bb(i)=bm
         END IF
      END DO

c ..... Ordering solutions

      DO i=2,n_sol
         IF (chi2sol(i).lt.chi2sol(i-1)) THEN
            DO j=i-1,1,-1
               IF (chi2sol(j+1).lt.chi2sol(j)) THEN
                  dummy=chi2sol(j+1)
                  chi2sol(j+1)=chi2sol(j)
                  chi2sol(j)=dummy
                  dummy=zphot(j+1)
                  zphot(j+1)=zphot(j)
                  zphot(j)=dummy
                  idum=izphot(j+1)
                  izphot(j+1)=izphot(j)
                  izphot(j)=idum
                  ddummy=bb(j+1)
                  bb(j+1)=bb(j)
                  bb(j)=ddummy
               END IF
            END DO
         END IF
      END DO

c ..... Check absolute magnitude of the first solution
c       if amagb not in [amabs_1,amabs_2] search the next 
c       solution satisfying constraint
      DO i=1,nz
         iforbid(i)=0
      END DO
      iz=izphot(1)
c absolute magnitude

      do ii=1,nfmab
      amagb(ii)=abmag2(ii,iz)
      enddo

      amagb(1)=abmag(iz)    !this is the old single M_lambda

c constraint absolute magnitude
c....... SOLUTION MAIN IS WITHIN M_abs WINDOW:
      IF (amagb(1).ge.amabs_1.and.amagb(1).le.amabs_2) GO TO 2000 ! solution ok

c....... SOLUTION MAIN IS OUTSIDE M_abs WINDOW:
c first solution forbidden, search the next one
      iforbid(iz)=1
 200  chi2min=1.e6
      DO i=i1,i2
         IF (z_min_array(i).lt.chi2min.and.iforbid(i).ne.1) THEN
            chi2min=z_min_array(i)
            iz=i
         END IF
      END DO

      DO i=1,n_sol
         IF (iz.eq.izphot(i)) THEN 
c check the absolute magnitude for the i-th solution

      do ii=1,nfmab
      amagb(ii)=abmag2(ii,iz)
      enddo

            amagb(1)=abmag(iz)
c constraint absolute magnitude
c i-th solution ok -> re-order solutions
            IF (amagb(1).ge.amabs_1.and.amagb(1).le.amabs_2) THEN
               GO TO 3000
            ELSE
c i-th solution forbidden
               iforbid(iz)=1
               GO TO 200
            END IF 
         END IF
      END DO

c refinement if the new minimum is not a secondary solution
      IF (iz.eq.i1) THEN
         dz_f=(z(iz+1)-z(iz))/10.
         zmax_f=z(iz+1)
         zmin_f=z(iz)
      ELSE IF (iz.eq.i2) THEN
         dz_f=(z(iz)-z(iz-1))/10.
         zmax_f=z(iz)
         zmin_f=z(iz-1)
      ELSE
         dz_f=(z(iz+1)-z(iz-1))/20.
         zmax_f=z(iz+1)
         zmin_f=z(iz-1)
      END IF
      IF (zmin_f.lt.z(i1)) zmin_f=z(i1)
      IF (zmax_f.gt.z(i2)) zmax_f=z(i2)
      av=avmin+(i_red(iz)-1)*averr
      fact_lyf=r_lyf**(i_lyf(iz)-1)*(1./r_lyf)
      CALL zfit_min(i_age(iz),i_typ(iz),av,
     .     zmin_f,zmax_f,dz_f,chi2,zop)
c check absolute magnitude
      do ii=1,nfmab
      amagb(ii)=abmag2(ii,iz)
      enddo

      amagb(1)=abmag(iz)
      IF (amagb(1).ge.amabs_1.and.amagb(1).le.amabs_2) THEN
c found new first solution, add solution and shift the others
         n_sol=n_sol+1
         DO i=n_sol,2,-1
            chi2sol(i)=chi2sol(i-1)
            zphot(i)=zphot(i-1)
            izphot(i)=izphot(i-1)
            bb(i)=bb(i-1)
         END DO
         zphot(1)=zop
         chi2sol(1)=chi2
         bb(1)=bm
         izphot(1)=iz
         GO TO 2000
      ELSE 

c... Is this the LAST possible point?
      ii=0
      DO i=i1,i2
      if(iforbid(i).lt.1)ii=ii+1
      enddo
c         IF (iz.ne.i2) THEN
          IF (ii.gt.0) THEN
c forbidden solution, try again
            iforbid(iz)=1
            GO TO 200
         ELSE
            WRITE(*,'(1x,a,i9)') 'WARNING: impossible to find a '//
     .           'solution within constraints for object ', id
c ..... Write outputs in any case (except .m and .spe)
c ..... File .z_phot: at 13131: anomalous objects
            zphot(1)=-9.9

c ..... File .obs_sed
            IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
               IF (iunit_sed.eq.0) THEN
                  WRITE(15,fmt2) id,(fl1(i)*2.0,i=1,n_filter),
     .                 (err_fl(i)*2.0,i=1,n_filter)    !unites de E-17 (NOT 2.E-17)
               ELSE IF (iunit_sed.eq.1) THEN
                  DO k=1,n_filter
                     f_jy(k)=fl1(k)*2.E-17*
     .                    (wl(k)**2/(2.9979E+18))*1.E+29
                     err_jy(k)=(2.3/2.5)*f_jy(k)*err_p(k)
                  if(amag(k).gt.90..and.law_nd(k).eq.1)then
                     err_jy(k)=(2.3/2.5)*err_p(k)*
     .               fl_nd(k)*2.E-17*
     .               (wl(k)**2/(2.9979E+18))*1.E+29
                  endif
                  END DO
                  WRITE(15,fmt2) id,(f_jy(i),i=1,n_filter),
     .                 (err_jy(i),i=1,n_filter)
               ELSE IF (iunit_sed.eq.2) THEN
                  DO k=1,n_filter
                     amag(k)=amag(k)+conv_mag(k)
               IF (amag(k).gt.90.)amag(k)=mag_nd(k)+conv_mag(k)
                  END DO
                  WRITE(15,fmt1) id,(amag(i),i=1,n_filter),
     .                 (err_p(i),i=1,n_filter)
               END IF
            END IF

            GO TO 13131

         END IF
      END IF

c re-ordering
 3000 izmin=izphot(i)
      dummy=chi2sol(i)
      dummyz=zphot(i)
      idum=izphot(i)
      ddummy=bb(i)
      i_sol=i
      DO i=i_sol,2,-1
         chi2sol(i)=chi2sol(i-1)
         zphot(i)=zphot(i-1)
         izphot(i)=izphot(i-1)
         bb(i)=bb(i-1)
      END DO
      chi2sol(1)=dummy
      zphot(1)=dummyz
      izphot(1)=idum
      bb(1)=ddummy

c ..... main solution 
 2000 i_sol=1
      izmin=izphot(1)
      itypem=i_typ(izmin)
      iagem=i_age(izmin)
      iredm=i_red(izmin)
      av=avmin+(iredm-1)*averr
      ilyfm=i_lyf(izmin)
      fact_lyf=r_lyf**(ilyfm-1)*(1./r_lyf)
      DO k=1,n_filter
         IF (flux(k,izmin,iagem,itypem,iredm,ilyfm).eq.0..and.
     .        i_lyman(k).eq.1) THEN
            fl1(k)=0.
         ELSE 
            fl1(k)=fl(k)
         END IF
      END DO
      IF (sed_abc(i_typ(izmin)).eq.'BC'.or.sed_abc(i_typ(izmin)).eq.'S'
     .   .or.sed_abc(i_typ(izmin)).eq.'MA') THEN
         age=T(i_age(izmin))/1.e9
      ELSE IF (sed_abc(i_typ(izmin)).eq.'AS'.or.
     .        sed_abc(i_typ(izmin)).eq.'A ') THEN
         age=19.9999
      END IF
      r_chi2=chi2sol(1)/deg
      IF (r_chi2.ge.1.e4) r_chi2=9999.99
      qq=100.*qchi2(0.5*deg,0.5*chi2sol(1))      
      IF (z(izmin)-zphot(1).gt.dz_f/2.) THEN
         izmin1=izmin-1
      ELSE
         izmin1=izmin
      END IF

c ..... study of confidence intervals
      DO i=1,3
         zz1(i)=z(i1)
         zz2(i)=z(i2)
      END DO
      DO i=1,mxz
         q_pro1(i)=0.
         q_pro2(i)=0.
         q_proz(i)=0.
      END DO
      poids=0.0
      poids_z=0.0
      z_poids=0.
      qq_p=0.

c ..... Values after izmin
      z_b1(1)=zphot(1)
      chi2_b1(1)=chi2sol(1)
      q_pro1(1)=qq
      i=izmin1+1
      npoiz1=1
      IF (ABS(z(izmin1)-zphot(1)).lt.1.e-5.and.izmin1.eq.i2) THEN
         DO ii=1,3              ! z at P = 99 90 and 68%
            zz2(ii)=z(i2)
         END DO
         npoiz1=1
      ELSE
 1001    CONTINUE
         k=i-izmin1+1
         q_pro1(k)=q_pro(i)
         z_b1(k)=z(i)
         chi2_b1(k)=z_min_array(i)

         IF (i.lt.i2.and.i.le.izmin1) THEN
            i=i+1
            GO TO 1001
         ELSE IF (i.lt.i2.and.(chi2_b1(k)-chi2sol(1)).le.val(1)) THEN
            i=i+1
            GO TO 1001
         END IF
         npoiz1=k

         DO ii=1,3              ! z at P = 99 90 and 68%
            l=0
            zz2(ii)=rlin(val(ii)+chi2sol(1),chi2_b1,z_b1,npoiz1,l)
            IF (zz2(ii).lt.z(i1)) zz2(ii)=z(i1)
            IF (zz2(ii).gt.z(i2)) zz2(ii)=z(i2)
            IF (zz2(ii).lt.zphot(1)) zz2(ii)=z2
         END DO
      END IF

c ..... Values before izmin

      z_b2(1)=zphot(1)
      chi2_b2(1)=chi2sol(1)
      i=izmin1
      ipass=0
      npoiz2=1
      IF (ABS(z(izmin1)-zphot(1)).lt.1.e-5) THEN
         q_pro2(1)=q_pro(izmin1)
         ipass=1
      END IF
      IF (ABS(z(izmin1)-zphot(1)).lt.1.e-5.and.izmin1.eq.i1) THEN
         DO ii=1,3                 !z at P = 99 90 and 68%
            zz1(ii)=z(i1)
         END DO
         npoiz2=1
      ELSE
 1003    CONTINUE
         k=izmin1-i+2
         q_pro2(k)=q_pro(i-ipass)
         z_b2(k)=z(i-ipass)
         chi2_b2(k)=z_min_array(i-ipass)
         IF (i-ipass.gt.1.and.i-ipass.ge.izmin1) THEN
            i=i-1
            GO TO 1003
         ELSE IF (i-ipass.gt.1.and.
     .           (chi2_b2(k)-chi2sol(1)).le.val(1)) THEN
            i=i-1
            GO TO 1003
         END IF
         npoiz2=k

         DO ii=1,3                 !z at P = 99 90 and 68%
         l=0
            zz1(ii)=rlin(val(ii)+chi2sol(1),chi2_b2,z_b2,npoiz2,l)
            IF (zz1(ii).lt.z(i1)) zz1(ii)=z(i1)
            IF (zz1(ii).gt.z(i2)) zz1(ii)=z(i2)
            IF (zz1(ii).gt.zphot(1)) zz1(ii)=zphot(1)
         END DO
      END IF

c ..... Ordering q_proz et z_b

      IF (ABS(z(izmin1)-zphot(1)).lt.1.e-5.and.izmin1.eq.i1) THEN
         npoiz=npoiz1           ! points after only 
         DO ii=1,npoiz
            q_proz(ii)=q_pro1(ii)
            z_b(ii)=z_b1(ii)
         END DO         
      ELSE IF (ABS(z(izmin1)-zphot(1)).lt.1.e-5.and.izmin1.eq.i2) THEN
         npoiz=npoiz2           ! points before only  
         DO ii=1,npoiz
            q_proz(ii)=q_pro2(npoiz2-(ii-1))
            z_b(ii)=z_b2(npoiz2-(ii-1))
         END DO         
      ELSE IF (ABS(z(izmin1)-zphot(1)).lt.1.e-5.and.
     .        izmin1.ne.1.and.izmin1.ne.i2) THEN
         npoiz=npoiz1+npoiz2-1
         DO ii=1,npoiz2         ! points before
            q_proz(ii)=q_pro2(npoiz2-(ii-1))
            z_b(ii)=z_b2(npoiz2-(ii-1))
         END DO
         DO ii=1,npoiz1-1       ! points after (skip zphot(1))
            q_proz(npoiz2+ii)=q_pro1(ii+1)
            z_b(npoiz2+ii)=z_b1(ii+1)
         END DO
      ELSE
         npoiz=npoiz1+npoiz2-2
         DO ii=1,npoiz2-1       ! points before (skip zphot(1))
            q_proz(ii)=q_pro2(npoiz2-(ii-1))
            z_b(ii)=z_b2(npoiz2-(ii-1))
         END DO
         DO ii=1,npoiz1-1       ! points after (skip zphot(1))
            q_proz(npoiz2+ii-1)=q_pro1(ii+1)
            z_b(npoiz2+ii-1)=z_b1(ii+1)
         END DO
      END IF

      DO ii=1,npoiz
         poids=poids+q_proz(ii)
         poids_z=poids_z+q_proz(ii)*z_b(ii)
      END DO

c ..... z weighted mean

      IF (poids.gt.0.0) THEN
         z_poids=poids_z/poids
         l=0
         qq_p=rlin(z_poids,z_b,q_proz,npoiz,l)
      ELSE
         z_poids=zphot(1)
         qq_p=qq
      END IF

c ..... secondary solution
      IF (n_sol.lt.2) THEN      ! no secondary solution
         zphot(2)=9.999
         qq2=0.
      ELSE
         qq2=100.*qchi2(0.5*deg,0.5*chi2sol(2))
      END IF

c..... Integrated probability around z_best

c .... probability at z_best+/-0.1

        p_int=0.0 

        if (renor_log.gt.0.0)then

         DO i=1,nz-1
         if(zphot(1)-0.10.ge.z(i).and.zphot(1)-0.10.lt.z(i+1))
     .   iz_int10a=i
         if(zphot(1)+0.10.ge.z(i).and.zphot(1)+0.10.lt.z(i+1))
     .   iz_int10b=i+1
         END DO

c... Conditions limites:

         if(zphot(1)-0.10.lt.z(1))iz_int10a=1
         if(zphot(1)+0.10.gt.z(nz))iz_int10b=nz

         do k=1,nz
         IF (abmag(k).ge.amabs_1.and.abmag(k).le.amabs_2)then
         if(k.ge.iz_int10a.and.k.le.iz_int10b)
     .   p_int=p_int+proba_z(k)
         endif
         enddo

         p_int=p_int-
     .   abs(proba_z(iz_int10a)-proba_z(iz_int10a+1))
     .   *(zphot(1)-0.10-z(iz_int10a))/
     .   (z(iz_int10a+1)-z(iz_int10a))

         p_int=p_int-
     .   abs(proba_z(iz_int10b-1)-proba_z(iz_int10b))
     .   *(z(iz_int10b)-zphot(1)-0.10)/
     .   (z(iz_int10b)-z(iz_int10b-1))

         
         p_int=p_int*100.0/renor_log
         if(p_int.lt.1.e-10)p_int=0.0
         if(p_int.gt.100.0)p_int=100.0

         endif

c ..... Probability to be in the cluster (absolue values)
      IF (z_clus.ge.0.) THEN
         IF (z_clus.eq.z(iz_clus)) THEN
            p_clus=q_pro(iz_clus)
         ELSE
            p_clus=q_pro(iz_clus)+(q_pro(iz_clus+1)-q_pro(iz_clus))
     .           *(z_clus-z(iz_clus))/(z(iz_clus+1)-z(iz_clus))
         END IF

c .... Integrated probability at z_cluster+/-0.05, 0.1
         p05=0.0
         p10=0.0
         if (renor_log.gt.0.0)then

         do k=i1,i2
         IF (abmag(k).ge.amabs_1.and.abmag(k).le.amabs_2)then
         if(k.ge.iz_clus05a.and.k.le.iz_clus05b)p05=p05+proba_z(k)
         if(k.ge.iz_clus10a.and.k.le.iz_clus10b)p10=p10+proba_z(k)
         endif
         enddo
         p05=p05-abs(proba_z(iz_clus05a)-proba_z(iz_clus05a+1))
     .   *(z_clus-0.05-z(iz_clus05a))/(z(iz_clus05a+1)-z(iz_clus05a))

         p05=p05-abs(proba_z(iz_clus05b-1)-proba_z(iz_clus05b))
     .   *(z(iz_clus05b)-z_clus-0.05)/(z(iz_clus05b)-z(iz_clus05b-1))

         p10=p10-abs(proba_z(iz_clus10a)-proba_z(iz_clus10a+1))
     .   *(z_clus-0.10-z(iz_clus10a))/(z(iz_clus10a+1)-z(iz_clus10a))

         p10=p10-abs(proba_z(iz_clus10b-1)-proba_z(iz_clus10b))
     .   *(z(iz_clus10b)-z_clus-0.10)/(z(iz_clus10b)-z(iz_clus10b-1))

         p05=p05*100.0/renor_log
         p10=p10*100.0/renor_log
         if(p05.lt.1.e-10)p05=0.0
         if(p10.lt.1.e-10)p10=0.0
         if(p05.gt.100.0)p05=100.0
         if(p10.gt.100.0)p10=100.0
         endif

c.. Test
c         if(p10.lt.p05)then
c         write(*,*)id,renor_log
c         write(*,*)iz_clus05a,iz_clus05b,iz_clus10a,iz_clus10b
c         do k=iz_clus10a,iz_clus10b
c         write(*,*)z(k),q_pro(k)
c         enddo
c         read(*,'(A)')line
c         endif

c .... probability at z+/-0.1, within zp1 and zp2, dz=0.02=dzp

      IF (input(37).eq.1.and.input(38).eq.1)then
        do k=iz1,iz2
        p_Ediscs(k)=0.0
        z_Ediscs(k)=zp1+(k-1)*dz
        enddo

        if (renor_log.gt.0.0)then

         do j=iz1,iz2

         DO i=1,nz-1
         if(z_Ediscs(j)-0.10.ge.z(i).and.z_Ediscs(j)-0.10.lt.z(i+1))
     .   iz_top10a=i
         if(z_Ediscs(j)+0.10.ge.z(i).and.z_Ediscs(j)+0.10.lt.z(i+1))
     .   iz_top10b=i+1
         END DO

         do k=i1,i2
         IF (abmag(k).ge.amabs_1.and.abmag(k).le.amabs_2)then
         if(k.ge.iz_top10a.and.k.le.iz_top10b)
     .   p_Ediscs(j)=p_Ediscs(j)+proba_z(k)
         endif
         enddo

         p_Ediscs(j)=p_Ediscs(j)-
     .   abs(proba_z(iz_top10a)-proba_z(iz_top10a+1))
     .   *(z_Ediscs(j)-0.10-z(iz_top10a))/
     .   (z(iz_top10a+1)-z(iz_top10a))

         p_Ediscs(j)=p_Ediscs(j)-
     .   abs(proba_z(iz_top10b-1)-proba_z(iz_top10b))
     .   *(z(iz_top10b)-z_Ediscs(j)-0.10)/
     .   (z(iz_top10b)-z(iz_top10b-1))

         
         p_Ediscs(j)=p_Ediscs(j)*100.0/renor_log
         if(p_Ediscs(j).lt.1.e-10)p_Ediscs(j)=0.0
         if(p_Ediscs(j).gt.100.0)p_Ediscs(j)=100.0
         enddo

         endif
       endif

c ..... Absolute magnitude in the cluster

         DO k=1,iw(itypea)
            y(k)=f(k,iagea,itypea)
            ww(k)=w(k,itypea)
         END DO
         av_clus=avmin+(ireda-1)*averr
         IF (avmin+(ireda-1)*averr.gt.0.0) THEN
            CALL red_spec(ww,y,av_clus,yob,1,iw(itypea),law_rd)
            DO k=1,iw(itypea)
               y(k)=yob(k)
            END DO
         END IF

         fact_lyf=r_lyf**(ilyfa-1)*(1./r_lyf)
         CALL lyman_forest(ww,y,iw(itypea),z_clus)

      if(mscale.lt.1.5)then ! M_abs scaled to the closest filter

         bmag=amag(n_fil_wa)-d_mod(h0,om,ov,z_clus)+2.5*
     .        LOG10(filter(nfil(n_fil_wa),ww,y,iw(itypea),z_clus)/
     .        filter(nfil(n_fil_wa),ww,y,iw(itypea),0.))
         amagb_clus=bmag-zeropoint(n_fil_wa)
     .        +2.5*LOG10(filter(nfil(n_fil_wa),ww,y,iw(itypea),0.)/
     .        filter(nfilt_mabs(1),ww,y,iw(itypea),0.))
         IF (magtyp.eq.1)amagb_clus=amagb_clus+conv_mag(0)

      else   !mscale=2 => b * 2x10^(-17) * F_lambda(model)

         bmag=filter(nfil(0),w_vega,f_vega,iw_vega,0.)/
     .   filter(nfil(0),ww,y,iw(itypea),0.)
         amagb_clus=2.5*LOG10(bmag)-d_mod(h0,om,ov,z_clus)-
     .              2.5*LOG10(bb_array(iz_clus))
         IF (magtyp.eq.1)amagb_clus=amagb_clus+conv_mag(0)

      endif

        ENDIF    ! cluster mode

c -------------------------------------------------------------------------
c .. Best fit with a reference template at z=zphot(1)
c -------------------------------------------------------------------------

         if(n_files_ref.gt.0)then
         
         n_ref=0
         iz_ref=1+int((zphot(1)-zmin)/dz)  ! z top

         do i=1,n_files_ref
         xi2_ref(i)=999.0
         enddo

         do itype=1,n_files_ref    ! scan reference SEDs=> chi2

         sxx=0.D0
         sxy=0.D0
         b=0.D0
                     DO k=1,n_filter
                        IF (fl(k).ge.0.) THEN ! no oof objects
                        IF (flux_ref(k,iz_ref,itype,ilyfm).eq.0..and.
     .                      i_lyman(k).eq.1) THEN
                            fl1(k)=0.
                           ELSE
                            fl1(k)=fl(k)
                              wt=1.D0/(err_fl(k)**2)
                              sxx=sxx+
     .                             DBLE(flux_ref(k,iz_ref,itype,ilyfm))
     .                             *DBLE(flux_ref(k,iz_ref,itype,ilyfm))
     .                             *wt
                              sxy=sxy+DBLE(fl1(k))*wt*
     .                             DBLE(flux_ref(k,iz_ref,itype,ilyfm))
                           END IF
                        ELSE
                           fl1(k)=-1.
                        END IF
                     END DO
                     b=sxy/sxx
                     chi2=0.
                     DO k=1,n_filter
                        IF (fl1(k).ge.0.) THEN ! no oof obj.
                           chi2=chi2+((fl1(k)
     .                          -b*flux_ref(k,iz_ref,itype,ilyfm))
     .                          /err_fl(k))**2
                        END IF
                     END DO

         if((chi2-xi2_ref(itype)).lt.0.0)xi2_ref(itype)=chi2/deg ! reduced chi2
 
         enddo

         chi2=1.e6
         do i=1,n_files_ref
         if(xi2_ref(i).lt.chi2)then
         chi2=xi2_ref(i)
         n_ref=i
         endif
         if(xi2_ref(i).gt.999.0)xi2_ref(i)=999.0
         enddo
 
         endif

c -------------------------------------------------------------------------
13130    continue   ! all writting sequences are sent here for normal objects
c -------------------------------------------------------------------------

c ..... writing output files
         fact_lyf=r_lyf**(ilyfm-1)*(1./r_lyf)
c         b_scale=bb(1)*(2.0E-17)*4.0*pi*(1.0+zphot(1))*
c     .   (d_lum(h0,om,ov,zphot(1))*d_lum(h0,om,ov,zphot(1)))

c          b_scale=dlog10(bb(1)*(2.0D-17)*4.D0*DBLE(pi*(1.0+zphot(1))))+
          b_scale=dlog10(bb(1)*(2.0D-17)*4.D0*DBLE(pi))+
     .   (2.D0*dlog10(DBLE(d_lum(h0,om,ov,zphot(1)))))

        if (renor_log.gt.0.0)then
        do k=1,nz
        proba_z(k)=proba_z(k)*100.0/renor_log
        IF(abmag(k).lt.amabs_1.or.abmag(k).gt.amabs_2)proba_z(k)=0.0
        enddo
c         WRITE(21,fmtPDZ)id,(proba_z(k)*100.0/renor_log,k=1,nz)
        else
        do k=1,nz
        proba_z(k)=0.0
        enddo
        endif

        IF(cPDZ.eq.'y'.or.cPDZ.eq.'Y')
     .  WRITE(21,fmtPDZ)id,(proba_z(k),k=1,nz)

        if(n_files_ref.eq.0)then  ! no reference templates

         IF (z_clus.ge.0.) THEN
         IF(cz.eq.'y'.or.cz.eq.'Y')WRITE(17,fmt2009) id,zphot(1),r_chi2,
     .        qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),zz1(2),
     .        zz2(2),zz1(3),zz2(3),z_poids,qq_p,
     .       (amagb(i),i=1,nfmab),p_int,fact_lyf,
     .        zphot(2),qq2,amagb_clus,p_clus,amag(n_fil_wa),
     .        p05,p10,b_scale

      IF (input(37).eq.1.and.input(38).eq.1)
     .    WRITE(20,fmt6)id,p05,p10,(p_Ediscs(i),i=1,iz2)

      ELSE   ! IF not in z_cluster mode

       IF(cz.eq.'y'.or.cz.eq.'Y')WRITE(17,fmt2008) id,zphot(1),r_chi2,
     .        qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),zz1(2),zz2(2),
     .        zz1(3),zz2(3),z_poids,qq_p,(amagb(i),i=1,nfmab),p_int,
     .        fact_lyf,zphot(2),qq2,b_scale

      END IF

         else              ! if reference templates

         IF (z_clus.ge.0.) THEN
      IF(cz.eq.'y'.or.cz.eq.'Y')WRITE(17,fmt2009) id,zphot(1),r_chi2,
     .qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),zz1(2),zz2(2),
     .zz1(3),zz2(3),z_poids,qq_p,(amagb(i),i=1,nfmab),p_int,
     .fact_lyf,zphot(2),qq2,
     .amagb_clus,p_clus,amag(n_fil_wa),
     .p05,p10,n_ref,(xi2_ref(i),i=1,n_files_ref),b_scale

      IF (input(37).eq.1.and.input(38).eq.1)
     .    WRITE(20,fmt6)id,p05,p10,(p_Ediscs(i),i=1,iz2)

      ELSE   ! IF not in z_cluster mode

       IF(cz.eq.'y'.or.cz.eq.'Y')WRITE(17,fmt2008) id,zphot(1),r_chi2,
     .        qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),zz1(2),zz2(2),
     .        zz1(3),zz2(3),z_poids,qq_p,(amagb(i),i=1,nfmab),p_int,
     .        fact_lyf,zphot(2),qq2,n_ref,(xi2_ref(i),i=1,n_files_ref),
     .        b_scale

      END IF

         endif

      IF(ccat.eq.'y'.or.ccat.eq.'Y')WRITE(10,fmt2001)id,zphot(1),r_chi2,
     .     qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),zz1(2),zz2(2),
     .     zz1(3),zz2(3),z_poids,qq_p,(amagb(i),i=1,nfmab),p_int,
     .     fact_lyf,i_sol,b_scale

c.TEST PB memoire....

c      if(zphot(1).gt.6.0)then
c      write(*,*)nz,nlyf,n_filter,n_filter+nfmab,nred,n_files
c      write(*,*)id,zphot(1),r_chi2
c      write(*,*)id,(amag(i),i=1,n_filter)
c      write(*,*)id,(fl(i),i=1,n_filter)
c      read(*,'(A)')line
c      endif

c -------------------------------------------------------------------------

c ..... other solutions (if .cat_phot requested, check threshold)

      IF (ccat.eq.'y'.or.ccat.eq.'Y') THEN
         DO i_sol=2,n_sol
            qq=100.*qchi2(0.5*deg,0.5*chi2sol(i_sol))
            IF (qq.ge.pthresh) THEN
               izmin=izphot(i_sol)
       IF (sed_abc(i_typ(izmin)).eq.'BC'.or.sed_abc(i_typ(izmin)).eq.'S'
     .   .or.sed_abc(i_typ(izmin)).eq.'MA') THEN
                  age=T(i_age(izmin))/1.e9
               ELSE IF (sed_abc(i_typ(izmin)).eq.'AS'.or.
     .                 sed_abc(i_typ(izmin)).eq.'A ') THEN
                  age=19.9999
               END IF
               r_chi2=chi2sol(i_sol)/deg
               IF (r_chi2.ge.1.e4) r_chi2=9999.99
               qq=100.*qchi2(0.5*deg,0.5*chi2sol(i_sol))      
               IF (z(izmin)-zphot(i_sol).gt.dz_f/2.) THEN
                  izmin1=izmin-1
               ELSE
                  izmin1=izmin
               END IF

c ..... study of confidence intervals
               DO i=1,3
                  zz1(i)=z(i1)
                  zz2(i)=z(i2)
               END DO
               DO i=1,mxz
                  q_pro1(i)=0.
                  q_pro2(i)=0.
                  q_proz(i)=0.
               END DO
               poids=0.0
               poids_z=0.0
               z_poids=0.
               qq_p=0.

c ..... Values after izmin
               z_b1(1)=zphot(i_sol)
               chi2_b1(1)=chi2sol(i_sol)
               q_pro1(1)=qq
               i=izmin1+1
               npoiz1=1
               IF (ABS(z(izmin1)-zphot(i_sol)).lt.1.e-5.and.
     .              izmin1.eq.i2) THEN
                  DO ii=1,3     ! z at P = 99 90 and 68%
                     zz2(ii)=z(i2)
                  END DO
                  npoiz1=1
               ELSE
 1002             CONTINUE
                  k=i-izmin1+1
                  q_pro1(k)=q_pro(i)
                  z_b1(k)=z(i)
                  chi2_b1(k)=z_min_array(i)

                  IF (i.lt.i2.and.i.le.izmin1) THEN
                     i=i+1
                     GO TO 1002
                  ELSE IF (i.lt.i2.and.
     .                    (chi2_b1(k)-chi2sol(i_sol)).le.val(1)) THEN
                     i=i+1
                     GO TO 1002
                  END IF
                  npoiz1=k

                  DO ii=1,3     ! z at P = 99 90 and 68%
                     l=0
                     zz2(ii)=rlin(val(ii)+chi2sol(i_sol),
     .                    chi2_b1,z_b1,npoiz1,l)
                     IF (zz2(ii).lt.z(i1)) zz2(ii)=z(i1)
                     IF (zz2(ii).gt.z(i2)) zz2(ii)=z(i2)
                     IF (zz2(ii).lt.zphot(i_sol)) zz2(ii)=z2
                  END DO
               END IF

c ..... Values before izmin

               z_b2(1)=zphot(i_sol)
               chi2_b2(1)=chi2sol(i_sol)
               i=izmin1
               ipass=0
               npoiz2=1
               IF (ABS(z(izmin1)-zphot(i_sol)).lt.1.e-5) THEN
                  q_pro2(1)=q_pro(izmin1)
                  ipass=1
               END IF
               IF (ABS(z(izmin1)-zphot(i_sol)).lt.1.e-5.and.
     .              izmin1.eq.i1) THEN
                  DO ii=1,3
                     zz1(ii)=z(i1)
                  END DO
                  npoiz2=1
               ELSE
 1004             CONTINUE
                  k=izmin1-i+2
                  q_pro2(k)=q_pro(i-ipass)
                  z_b2(k)=z(i-ipass)
                  chi2_b2(k)=z_min_array(i-ipass)
                  IF (i-ipass.gt.1.and.i-ipass.ge.izmin1) THEN
                     i=i-1
                     GO TO 1004
                  ELSE IF (i-ipass.gt.1.and.
     .                    (chi2_b2(k)-chi2sol(i_sol)).le.val(1)) THEN
                     i=i-1
                     GO TO 1004
                  END IF
                  npoiz2=k

                  DO ii=1,3     !z at P = 99 90 and 68%
                     l=0
                     zz1(ii)=rlin(val(ii)+chi2sol(i_sol),
     .                    chi2_b2,z_b2,npoiz2,l)
                     IF (zz1(ii).lt.z(i1)) zz1(ii)=z(i1)
                     IF (zz1(ii).gt.z(i2)) zz1(ii)=z(i2)
                     IF (zz1(ii).gt.zphot(i_sol)) zz1(ii)=zphot(i_sol)
                  END DO
               END IF

c ..... Ordering q_proz et z_b

               IF (ABS(z(izmin1)-zphot(i_sol)).lt.1.e-5.
     .              and.izmin1.eq.i1) THEN
                  npoiz=npoiz1  ! points after only 
                  DO ii=1,npoiz
                     q_proz(ii)=q_pro1(ii)
                     z_b(ii)=z_b1(ii)
                  END DO         
               ELSE IF (ABS(z(izmin1)-zphot(i_sol)).lt.1.e-5.and.
     .                 izmin1.eq.i2) THEN
                  npoiz=npoiz2  ! points before only  
                  DO ii=1,npoiz
                     q_proz(ii)=q_pro2(npoiz2-(ii-1))
                     z_b(ii)=z_b2(npoiz2-(ii-1))
                  END DO         
               ELSE IF (ABS(z(izmin1)-zphot(i_sol)).lt.1.e-5.and.
     .                 izmin1.ne.1.and.izmin1.ne.i2) THEN
                  npoiz=npoiz1+npoiz2-1
                  DO ii=1,npoiz2 ! points before
                     q_proz(ii)=q_pro2(npoiz2-(ii-1))
                     z_b(ii)=z_b2(npoiz2-(ii-1))
                  END DO
                  DO ii=1,npoiz1-1 ! points after (skip zphot(i_sol))
                     q_proz(npoiz2+ii)=q_pro1(ii+1)
                     z_b(npoiz2+ii)=z_b1(ii+1)
                  END DO
               ELSE
                  npoiz=npoiz1+npoiz2-2
                  DO ii=1,npoiz2-1 ! points before (skip zphot(i_sol))
                     q_proz(ii)=q_pro2(npoiz2-(ii-1))
                     z_b(ii)=z_b2(npoiz2-(ii-1))
                  END DO
                  DO ii=1,npoiz1-1 ! points after (skip zphot(i_sol))
                     q_proz(npoiz2+ii-1)=q_pro1(ii+1)
                     z_b(npoiz2+ii-1)=z_b1(ii+1)
                  END DO
               END IF

               DO ii=1,npoiz
                  poids=poids+q_proz(ii)
                  poids_z=poids_z+q_proz(ii)*z_b(ii)
               END DO

c ..... absolute magnitude
               itypem=i_typ(izmin)
               iagem=i_age(izmin)
               iredm=i_red(izmin)
               av=avmin+(iredm-1)*averr
               ilyfm=i_lyf(izmin)
               fact_lyf=r_lyf**(ilyfm-1)*(1./r_lyf)
               zz=0.0

      if(mscale.lt.1.5)then

      do ii=1,nfmab
      sauce=flux(n_fil_abs(ii,izmin),izmin,iagem,itypem,iredm,ilyfm)
     ./flux(n_fil_abs(ii,izmin),0,iagem,itypem,iredm,ilyfm)

               IF (sauce.gt.0.0) bmag=amag(n_fil_abs(ii,izmin))+
     .              2.5*LOG10(sauce)-d_mod(h0,om,ov,zphot(i_sol)) 

          sauce=flux(n_filter+ii,0,iagem,itypem,iredm,ilyfm)*
     .    surf(n_filter+ii)
c       IF (sauce.gt.0.0) amagb(ii)=bmag-(zeropoint(n_fil_abs(ii,izmin))-  !! bug v<9
       IF (sauce.gt.0.0) amagb(ii)=bmag+color(nfil(n_fil_abs(ii,izmin)),
     .  nfilt_mabs(ii),w_vega,f_vega,iw_vega,zz)+
     . 2.5*LOG10(flux(n_fil_abs(ii,izmin),0,iagem,itypem,iredm,
     .    ilyfm)*surf(n_fil_abs(ii,izmin))/sauce) 
          IF (magtyp.eq.1) amagb(ii)=amagb(ii)+conv_mag(n_filter+ii)

        enddo
     
        else   !mscale=2 => b * 2x10^(-17) * F_lambda(model)


      do ii=1,nfmab

      sauce=filter(nfil(n_filter+ii),w_vega,f_vega,iw_vega,zz)/
     .(surf(n_filter+ii)*
     . flux(n_filter+ii,0,iagem,itypem,iredm,ilyfm))
       IF (sauce.gt.0.0)amagb(ii)=2.5*LOG10(sauce)
     . -d_mod(h0,om,ov,zphot(i_sol))-2.5*LOG10(bb_array(i_sol))

          IF (magtyp.eq.1) amagb(ii)=amagb(ii)+conv_mag(n_filter+ii)
      enddo
       
        endif 

c ..... z weighted mean
               IF (poids.gt.0.0) THEN
                  z_poids=poids_z/poids
                  l=0
                  qq_p=rlin(z_poids,z_b,q_proz,npoiz,l)
               ELSE 
                  z_poids=zphot(i_sol)
                  qq_p=qq
               END IF

c..... Integrated probability around z_best

c .... probability at z_best(i_sol)+/-0.1

        p_int=0.0 

        if (renor_log.gt.0.0)then

         DO i=1,nz-1
         if(zphot(i_sol)-0.10.ge.z(i).and.zphot(i_sol)-0.10.lt.z(i+1))
     .   iz_clus10a=i
         if(zphot(i_sol)+0.10.ge.z(i).and.zphot(i_sol)+0.10.lt.z(i+1))
     .   iz_clus10b=i+1
         END DO

         do k=1,nz
         IF (abmag(k).ge.amabs_1.and.abmag(k).le.amabs_2)then
         if(k.ge.iz_clus10a.and.k.le.iz_clus10b)
     .   p_int=p_int+proba_z(k)
         endif
         enddo

         p_int=p_int-
     .   abs(proba_z(iz_clus10a)-proba_z(iz_clus10a+1))
     .   *(zphot(i_sol)-0.10-z(iz_clus10a))/
     .   (z(iz_clus10a+1)-z(iz_clus10a))

         p_int=p_int-
     .   abs(proba_z(iz_clus10b-1)-proba_z(iz_clus10b))
     .   *(z(iz_clus10b)-zphot(i_sol)-0.10)/
     .   (z(iz_clus10b)-z(iz_clus10b-1))
         
         p_int=p_int*100.0/renor_log
         if(p_int.lt.0.0)p_int=0.0
         if(p_int.gt.100.0)p_int=100.0

         endif

c         b_scale=bb(i_sol)*2.0E-17*4.0*pi*(1.0+zphot(i_sol))*
c     .  (d_lum(h0,om,ov,zphot(i_sol))*d_lum(h0,om,ov,zphot(i_sol)))

c         b_scale=dlog10(bb(i_sol)*2.0D-17*4.D0*
c     .   DBLE(pi*(1.0+zphot(i_sol))))+
c     .  (2.D0*dlog10(DBLE(d_lum(h0,om,ov,zphot(i_sol)))))

         b_scale=dlog10(bb(i_sol)*2.0D-17*4.D0*DBLE(pi))+
     .  (2.D0*dlog10(DBLE(d_lum(h0,om,ov,zphot(i_sol)))))

               WRITE(10,fmt2001) id,zphot(i_sol),r_chi2,qq,itypem,iagem,
     .              age,av,bb(i_sol),zz1(1),zz2(1),zz1(2),zz2(2),
     .              zz1(3),zz2(3),z_poids,qq_p,(amagb(i),i=1,nfmab),
     .              p_int,fact_lyf,i_sol,b_scale
            END IF
         END DO
      END IF

c -----------------------------------------------------------------------
c                    Graphics: Synthetic spectrum at zphot
c -----------------------------------------------------------------------

c ..... Write SED of object 'id'

      izmin=izphot(1)
      itypem=i_typ(izmin)
      iagem=i_age(izmin)
      iredm=i_red(izmin)
      av=avmin+(iredm-1)*averr
      ilyfm=i_lyf(izmin)
      fact_lyf=r_lyf**(ilyfm-1)*(1./r_lyf)

      IF (iunit_sed.eq.1) THEN
         DO k=1,n_filter
            f_jy(k)=fl1(k)*2.E-17*(wl(k)**2/(2.9979E+18))*1.E+29
            err_jy(k)=(2.3/2.5)*f_jy(k)*err_p(k)
                  if(amag(k).gt.90..and.law_nd(k).eq.1)then
                     err_jy(k)=(2.3/2.5)*err_p(k)*
     .               fl_nd(k)*2.E-17*
     .               (wl(k)**2/(2.9979E+18))*1.E+29
                  endif
         END DO
      END IF
      IF (iunit_sed.eq.2) THEN
c ..... Vega mag -> AB mag
         DO k=1,n_filter
            amag(k)=amag(k)+conv_mag(k)
            IF (amag(k).gt.90.) amag(k)=mag_nd(k)+conv_mag(k)
         END DO
      END IF
      IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
         IF (iunit_sed.eq.0) THEN
            WRITE(15,fmt2) id,(fl1(i)*2.0,i=1,n_filter),
     .           (err_fl(i)*2.0,i=1,n_filter)   !Units of 1.E-17 (NOT 1.E-17)
         ELSE IF (iunit_sed.eq.1) THEN
            WRITE(15,fmt2) id,(f_jy(i),i=1,n_filter),
     .           (err_jy(i),i=1,n_filter)
         ELSE IF (iunit_sed.eq.2) THEN
            WRITE(15,fmt1) id,(amag(i),i=1,n_filter),
     .           (err_p(i),i=1,n_filter)
         END IF
      END IF

      IF (csed2.eq.'y'.or.csed2.eq.'Y'.or.iout_s.gt.1) THEN 
         DO k=1,iw(itypem)
            y(k)=f(k,iagem,itypem)
            ww(k)=w(k,itypem)
         END DO
         av=avmin+(iredm-1)*averr
         IF (av.gt.0.0) THEN
            CALL red_spec(ww,y,av,yob,1,iw(itypem),law_rd)
            DO k=1,iw(itypem)  
               y(k)=yob(k)
            END DO
         END IF
         CALL lyman_forest(ww,y,iw(itypem),zphot(1))
      END IF

      IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
         DO k=1,n_filter
            fl_temp(k)=filter(nfil(k),ww,y,iw(itypem),zphot(1))/surf(k)
         END DO
         IF (iunit_sed.eq.0) THEN
            WRITE(19,fmt4) id,(bb(1)*fl_temp(k)*2.0,k=1,n_filter)
         ELSE IF (iunit_sed.eq.1) THEN
            WRITE(19,fmt4) id,(bb(1)*2.E-17*fl_temp(k)*
     .     (wl(k)**2/(2.9979E+18))*1.E+29,k=1,n_filter)
         ELSE IF (iunit_sed.eq.2) THEN
            DO k=1,n_filter
               IF (fl_temp(k).gt.0.) THEN
                  mag_temp(k)=-2.5*LOG10(bb(1)*fl_temp(k)*
     .                 surf(k)/vega(k))+conv_mag(k)
               ELSE
                  mag_temp(k)=100.
               END IF
            END DO
            WRITE(19,fmt3) id,(mag_temp(k),k=1,n_filter)
         END IF
      END IF

      IF (iout_s.gt.1) THEN
         spectra=root(1:iroot)//'.spe'

         IF (iunit_sed.eq.0) THEN
            DO k=1,iw(itypem)
               y(k)=bb(1)*y(k)*2.0/(1.+zphot(1))   !units of 10^-17 erg/s/cm2/A
            END DO
         ELSE IF (iunit_sed.eq.1) THEN
            DO k=1,iw(itypem)
               y(k)=bb(1)*y(k)*2.E-17*(ww(k)**2/2.9979E+18)*1.E+29*
     .              (1.+zphot(1))
            END DO
         ELSE IF (iunit_sed.eq.2) THEN
            DO k=1,iw(itypem)
               y(k)=bb(1)*y(k)*2.E-17*(ww(k)**2/2.9979E+18)*
     .              (1.+zphot(1))
               IF (y(k).ne.0.) THEN
                  y(k)=-2.5*LOG10(y(k))-48.60
               ELSE
                  y(k)=100.
               END IF
            END DO
         END IF
         iwav=0
         last_k=0
         wmin1=1.e5
         wmax1=0.
         DO k=1,n_filter
            wmin0=wl(k)-surf(k)/2.-1000.
            wmax0=wl(k)+surf(k)/2.+1000.
            IF (wmin0.lt.wmin1) wmin1=wmin0
            IF (wmax0.gt.wmax1) wmax1=wmax0
         END DO
         DO k=1,iw(itypem)
            w0=ww(k)*(1.+zphot(1))
            IF ((w0.ge.wmin1.and.w0.le.wmax1).or.
     .           (w0.le.wmin1.and.w0.ge.wmax1)) THEN
               iwav=iwav+1
               wsyn(iwav)=ww(k)*(1.+zphot(1))
               ysyn(iwav)=y(k)
               last_k=k
            END IF
         END DO

c RP (undersampling correction):
         IF(ww(last_k).lt.ww(iw(itypem)))THEN
         iwav=iwav+1
         wsyn(iwav)=ww(last_k+1)*(1.+zphot(1))
         ysyn(iwav)=y(last_k+1)
         END IF
c END (undersampling correction)

         OPEN(12,FILE=spectra,STATUS='unknown')
         DO k=1,iwav
            WRITE(12,*) wsyn(k),ysyn(k)
         END DO
         CLOSE(12)
      END IF

      GO TO 13400
c -------------------------------------------------------------------------
c -------------------------------------------------------------------------
13131 continue  ! all writting sequences are sent here for abnormal objects
c -------------------------------------------------------------------------

            r_chi2=99.
            qq=0.
            itypem=0
            iagem=0
            age=99.
            av=0.
            bb(1)=0.
            b_scale=0.D0
            DO i=1,3
               zz1(i)=99.
               zz2(i)=99.
            END DO
            z_poids=99.
            qq_p=0.
              do i=1,nfmab
              amagb(i)=99.
              enddo
            zphot(2)=99.
            qq2=0.
            amagb_clus=99.
            p_clus=0.
            amag(1)=99.
            i_sol=1
            fact_lyf=1.
            p05=0.0
            p10=0.0
            p_int=0.0
              do i=1,iz2
              p_Ediscs(i)=0.0
              enddo
              do i=1,nz
              proba_z(i)=0.0
              enddo

        IF (cPDZ.eq.'y'.or.cPDZ.eq.'Y')       
     .  WRITE(21,fmtPDZ)id,(proba_z(i),i=1,nz)  !CFHTLS PDZ

        if(n_files_ref.eq.0)then  ! no reference templates

            IF (z_clus.ge.0.) THEN
               IF(cz.eq.'y'.or.cz.eq.'Y')WRITE(17,fmt2009) id,zphot(1),
     .              r_chi2,qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),
     .              zz1(2),zz2(2),zz1(3),zz2(3),z_poids,qq_p,
     .              (amagb(i),i=1,nfmab),p_int,
     .              fact_lyf,zphot(2),qq2,amagb_clus,p_clus,amag(1),
     .              p05,p10,b_scale
          IF (input(37).eq.1.and.input(38).eq.1)
     .    WRITE(20,fmt6)id,p05,p10,(p_Ediscs(i),i=1,iz2)

          ELSE
              IF(cz.eq.'y'.or.cz.eq.'Y')WRITE(17,fmt2008) id,zphot(1),
     .              r_chi2,qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),
     .              zz1(2),zz2(2),zz1(3),zz2(3),z_poids,qq_p,
     .              (amagb(i),i=1,nfmab),p_int,
     .              fact_lyf,zphot(2),qq2,b_scale
          END IF

         else         ! if reference templates
         n_ref=0
         do i=1,n_files_ref
         xi2_ref(i)=999.0
         enddo

            IF (z_clus.ge.0.) THEN
               IF(cz.eq.'y'.or.cz.eq.'Y')WRITE(17,fmt2009) id,zphot(1),
     .              r_chi2,qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),
     .              zz1(2),zz2(2),zz1(3),zz2(3),z_poids,qq_p,
     .              (amagb(i),i=1,nfmab),p_int,
     .              fact_lyf,zphot(2),qq2,amagb_clus,p_clus,amag(1),
     .              p05,p10,n_ref,(xi2_ref(i),i=1,n_files_ref),b_scale
          IF (input(37).eq.1.and.input(38).eq.1)
     .    WRITE(20,fmt6)id,p05,p10,(p_Ediscs(i),i=1,iz2)

          ELSE
              IF(cz.eq.'y'.or.cz.eq.'Y')WRITE(17,fmt2008) id,zphot(1),
     .       r_chi2,qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),
     .       zz1(2),zz2(2),zz1(3),zz2(3),z_poids,qq_p,
     .       (amagb(i),i=1,nfmab),p_int,
     .       fact_lyf,zphot(2),qq2,n_ref,(xi2_ref(i),i=1,n_files_ref),
     .       b_scale
          END IF

         endif

c ..... File .cat_phot
         IF (ccat.eq.'y'.or.ccat.eq.'Y') WRITE(10,fmt2001) id,zphot(1),
     .           r_chi2,qq,itypem,iagem,age,av,bb(1),zz1(1),zz2(1),
     .           zz1(2),zz2(2),zz1(3),zz2(3),z_poids,qq_p,
     .           (amagb(i),i=1,nfmab),p_int,
     .           fact_lyf,i_sol,b_scale

c ..... File .temp_sed
            IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN
               DO k=1,n_filter
                  mag_temp(k)=99.
                  fl_temp(k)=-1.
               END DO
               IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
                  WRITE(19,fmt4) id,(fl_temp(k),k=1,n_filter)
               ELSE IF (iunit_sed.eq.2) THEN
                  WRITE(19,fmt3) id,(mag_temp(k),k=1,n_filter)
               END IF
            END IF

c.......................................................................
      GO TO 13400
13500 CONTINUE
c.......................................................................
c    Writing header 

c ... Writing file.header_zphot :
      WRITE(13,'(a)')'# c1 : Id.'
      WRITE(13,'(a)')'# c2 : Best z_phot'
      WRITE(13,'(a)')'# c3 : xi2'
      WRITE(13,'(a)')'# c4 : P(xi2)'
      WRITE(13,'(a)')'# c5 : Best-fit type'
      WRITE(13,'(a)')'# c6 : N(id) age '
      WRITE(13,'(a)')'# c7 : age [Gyr] '
      WRITE(13,'(a)')'# c8 : Av [mags] '
      WRITE(13,'(a)')'# c9 : Normalization constant '
      WRITE(13,'(a)')'# c10 : z_min 99% confidence'
      WRITE(13,'(a)')'# c11 : z_max 99% confidence'
      WRITE(13,'(a)')'# c12 : z_min 90% confidence'
      WRITE(13,'(a)')'# c13 : z_max 90% confidence'
      WRITE(13,'(a)')'# c14 : z_min 68% confidence'
      WRITE(13,'(a)')'# c15 : z_max 68% confidence'
      WRITE(13,'(a)')'# c16 : z_m mean weighted photo_z'
      WRITE(13,'(a)')'# c17 : P(z_m) mean weighted photo_z'
      do i=1,nfmab
      WRITE(13,'(a,i2,a,i2)')'# c',i+17,' : M_abs in filter ',i
      enddo
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+1,' : P_int(z_best+/-0.1)'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+2,' : Best Lyman forest'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+3,' : z_2 secondary solution'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+4,' : P(z_2) secondary z'


        if(n_files_ref.eq.0)then  ! no reference templates

         IF (z_clus.ge.0.) THEN
         IF(cz.eq.'y'.or.cz.eq.'Y')then
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+5,' : M_abs(ref) if in cluster'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+6,' : Proba of cluster member'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+7,' : app. mag for M_abs(ref)'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+8,' : P_int(z_cluster+/-0.05)'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+9,' : P_int(z_cluster+/-0.1)'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+10,' : b_scale factor'
         endif

      ELSE   ! IF not in z_cluster mode

         IF(cz.eq.'y'.or.cz.eq.'Y')then
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+5,' : b_scale factor'
         endif

      END IF

         else              ! if reference templates

         IF (z_clus.ge.0.) THEN

         IF(cz.eq.'y'.or.cz.eq.'Y')then
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+5,' : M_abs(ref) if in cluster'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+6,' : Proba of cluster member'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+7,' : app. mag for M_abs(ref)'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+8,' : P_int(z_cluster+/-0.05)'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+9,' : P_int(z_cluster+/-0.1)'
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+10,' : N(id) best fit template'
      do i=1,n_files_ref
      WRITE(13,'(a,i2,a,i2)')'# c',nfmab+17+10+i,': xi2 for template',i
      enddo
      WRITE(13,'(a,i2,a)')'# c',nfmab+28+n_files_ref,': b_scale factor'
         endif

      ELSE   ! IF not in z_cluster mode

         IF(cz.eq.'y'.or.cz.eq.'Y')then
      WRITE(13,'(a,i2,a)')'# c',nfmab+17+5,': N(id) best fit template'
      do i=1,n_files_ref
      WRITE(13,'(a,i2,a,i2)')'# c',nfmab+17+5+i,': xi2 for template',i
      enddo
      WRITE(13,'(a,i2,a)')'# c',nfmab+23+n_files_ref,': b_scale factor'
         endif

      END IF

         endif
c.......................................................................

      CLOSE(16)
      IF (ccat.eq.'y'.or.ccat.eq.'Y') CLOSE(10)
      IF (clog.eq.'y'.or.clog.eq.'Y') CLOSE(14)
      IF (csed1.eq.'y'.or.csed1.eq.'Y') CLOSE(15)
      IF (cz.eq.'y'.or.cz.eq.'Y') CLOSE(17)
      IF (csed2.eq.'y'.or.csed2.eq.'Y') CLOSE(19)
      close(20)
      IF (cPDZ.eq.'y'.or.cPDZ.eq.'Y') close(21)  ! CFHTLS PDZ file
      CLOSE(13)   ! header file

      time_tot=cputime()-time_tot
      WRITE(*,'(a)') ' Done !!!'
      WRITE(*,'(1X,A,F12.3,A)') 'Total CPU time involved : ', time_tot, 
     .     ' seconds'
      WRITE(18,'(1X,A,F12.3,A)') '#  Total CPU time involved : ',  
     .     time_tot,' seconds'
      CLOSE(18)
      STOP
 2002 FORMAT(i10,1x,f6.3,1x,e11.4,1x,f10.5,1x,i2,1x,f8.4,1x,F4.2,1x,
     .     F6.3,1x,f7.2)
 2012 FORMAT(i10,1x,f5.2,1x,i2,1x,i3,1x,f8.4,1x,f5.2,
     .4(1x,E11.4))
 25   STOP 'Error opening synthetic spectra files'
 78   STOP 'Error opening templates file'
 79   STOP 'Error opening or reading photometric catalog'
 80   STOP 'Error opening filters file'
 81   STOP 'Error opening output files'
 82   STOP 'Error reading filters file'
      END
c----------------------------------------------------------------------
      FUNCTION func(x)
      IMPLICIT NONE
      REAL func,pchi2,x,deg,prob
      EXTERNAL pchi2
      COMMON/nfree/deg,prob
c
      func=prob-pchi2(0.5*deg,0.5*x)
      RETURN
      END
c----------------------------------------------------------------------
      FUNCTION pro_z(a,x)
c     P(z) probability distribution..........
c     We use ~exp (-x/2), thus dof~2, in agreement with MC simus
      IMPLICIT NONE
      REAL a,x,b,pro_z
      INTEGER n      
      n=INT(a/2.0)
c      if(n.le.1)then
      if(x.lt.100)then
      pro_z=exp(-x/2.0)
      else
      pro_z=0.0
      endif
c      else
c      pro_z=(x**((a/2.0)-1.0))*(exp(-x/2.0))
c      endif

      RETURN
      END

c----------------------------------------------------------------------
c ..... Distance module: d_mod = 5*LOG10(d_L[pc]/10)
 
      FUNCTION d_mod(h0,om,ov,za)
      IMPLICIT NONE
      INTEGER nstep,i
      REAL d_mod,h0,om,ov,z,za,c,dmin,d_l,ee,s,s1,s2,dz,a,b,x
      PARAMETER (c=2.9979E+5,nstep=100)
      ee(x)=SQRT(om*(1.+x)**3+(1.-om-ov)*(1.+x)**2+ov)
c
      IF (za.lt.1.e-5) THEN
         z=1.e-5
      ELSE
         z=za
      END IF
      IF (om.lt.0.001.and.ov.eq.0.) THEN
         d_l=(1.+z)*LOG(1.+z)
      ELSE IF (om.eq.1..and.ov.eq.0.) THEN
         d_l=2.*(1.+z)*(1.-1./SQRT(1.+z))
      ELSE IF (om.lt.1..and.om.gt.0.01.and.ov.eq.0.) THEN
         d_l=(2./om**2)*(om*z-(2.-om)*(-1.+SQRT(om*z+1.))) ! Mattig
      ELSE
         dz=z/FLOAT(nstep)
         s=0.
         d_l=0.
         DO i=1,nstep+1
            a=(i-1)*dz
            b=i*dz
            s1=(1/ee(a))
            s2=(1/ee(b))
            s=dz*(s1+s2)/2.
            d_l=d_l+s
         END DO
         d_l=(1.+z)*d_l
      END IF
      d_mod=5.*LOG10((c/h0)*d_l)+25.
c   minimum d_mod: corresponding to d(~M31)=1000 Kpc
      dmin=25.
      IF (d_mod.lt.dmin) d_mod=dmin
      RETURN
      END
c ----------------------------------------------------------------------
c----------------------------------------------------------------------
c   Luminosity distance in cm .......................................

      FUNCTION d_lum(h0,om,ov,za)

      INTEGER nstep,i
      REAL d_mod,h0,om,ov,z,za,c,dmin,d_l,ee,s,s1,s2,dz,a,b,x
      PARAMETER (c=2.9979E+5,nstep=1000)
      ee(x)=SQRT(om*(1.+x)**3+(1.-om-ov)*(1.+x)**2+ov)
c
      IF (za.lt.1.e-5) THEN
         z=1.e-5
      ELSE
         z=za
      END IF
      IF (om.lt.0.001.and.ov.eq.0.) THEN
         d_l=(1.+z)*LOG(1.+z)
      ELSE IF (om.eq.1..and.ov.eq.0.) THEN
         d_l=2.*(1.+z)*(1.-1./SQRT(1.+z))
      ELSE IF (om.lt.1..and.om.gt.0.01.and.ov.eq.0.) THEN
         d_l=(2./om**2)*(om*z-(2.-om)*(-1.+SQRT(om*z+1.))) ! Mattig
      ELSE
         dz=z/FLOAT(nstep)
         s=0.
         d_l=0.
         DO i=1,nstep+1
            a=(i-1)*dz
            b=i*dz
            s1=(1/ee(a))
            s2=(1/ee(b))
            s=dz*(s1+s2)/2.
            d_l=d_l+s
         END DO
         d_l=(1.+z)*d_l
      END IF
      d_mod=5.*LOG10((c/h0)*d_l)+25.
      d_lum=10.*(10.0**(d_mod/5.))    !pc
      d_lum=d_lum*3.08568e+18   !cm
      RETURN
      END
c----------------------------------------------------------------------





